@@ -5,7 +5,6 @@
 # Patches by Mike Poidinger to support multiple databases.
 # Updated by Peter Cock in 2007 to do a better job on BLAST 2.2.15
 
-
 """
 This module provides code to work with the standalone version of
 BLAST, either blastall or blastpgp, provided by the NCBI.
@@ -35,7 +34,6 @@ rpsblast        Execute rpsblast.
 
 """
 
-from __future__ import generators
 import os
 import re
 
@@ -173,6 +171,13 @@ class _Scanner:
         attempt_read_and_call(uhandle, consumer.reference, start="RID:")
         read_and_call_while(uhandle, consumer.noevent, blank=1)
 
+        # blastpgp may have a reference for compositional score matrix
+        # adjustment (see Bug 2502):
+        if attempt_read_and_call(
+            uhandle, consumer.reference, start="Reference"):
+            read_and_call_until(uhandle, consumer.reference, blank=1)
+            read_and_call_while(uhandle, consumer.noevent, blank=1)
+
         # blastpgp has a Reference for composition-based statistics.
         if attempt_read_and_call(
             uhandle, consumer.reference, start="Reference"):
@@ -180,7 +185,7 @@ class _Scanner:
             read_and_call_while(uhandle, consumer.noevent, blank=1)
 
         line = uhandle.peekline()
-        assert line.strip() <> ""
+        assert line.strip() != ""
         assert not line.startswith("RID:")
         if line.startswith("Query=") :
             #This is an old style query then database...
@@ -260,8 +265,8 @@ class _Scanner:
         # blastpgp 2.0.10 from NCBI 9/19/99 for Solaris sometimes crashes here.
         # If this happens, the handle will yield no more information.
         if not uhandle.peekline():
-            raise ValueError, "Unexpected end of blast report.  " + \
-                  "Looks suspiciously like a PSI-BLAST crash."
+            raise ValueError("Unexpected end of blast report.  " + \
+                  "Looks suspiciously like a PSI-BLAST crash.")
 
         # BLASTN 2.2.3 sometimes spews a bunch of warnings and errors here:
         # Searching[blastall] WARNING:  [000.000]  AT1G08320: SetUpBlastSearch 
@@ -405,7 +410,7 @@ class _Scanner:
                 break
             elif is_blank_line(line):
                 # Check to make sure I haven't missed the Length line
-                raise ValueError, "I missed the Length in an alignment header"
+                raise ValueError("I missed the Length in an alignment header")
             consumer.title(line)
 
         # Older versions of BLAST will have a line with some spaces.
@@ -845,7 +850,7 @@ class _DescriptionConsumer:
 
     def round(self, line):
         if not line.startswith('Results from round'):
-            raise ValueError, "I didn't understand the round line\n%s" % line
+            raise ValueError("I didn't understand the round line\n%s" % line)
         self._roundnum = _safe_int(line[18:].strip())
 
     def end_descriptions(self):
@@ -864,8 +869,8 @@ class _DescriptionConsumer:
         #   - sometimes there's an "N" score of '1'.
         cols = line.split()
         if len(cols) < 3:
-            raise ValueError, \
-                  "Line does not appear to contain description:\n%s" % line
+            raise ValueError( \
+                  "Line does not appear to contain description:\n%s" % line)
         if self.__has_n:
             i = line.rfind(cols[-1])        # find start of N
             i = line.rfind(cols[-2], 0, i)  # find start of p-value
@@ -915,8 +920,7 @@ class _AlignmentConsumer:
             try:
                 name, start, seq, end = line.split()
             except ValueError:
-                raise ValueError, "I do not understand the line\n%s" \
-                      % line
+                raise ValueError("I do not understand the line\n%s" % line)
             self._start_index = line.index(start, len(name))
             self._seq_index = line.index(seq,
                                          self._start_index+len(start))
@@ -1073,18 +1077,21 @@ class _HSPConsumer:
             r"Identities = (\d+)\/(\d+)", line,
             "I could not find the identities in line\n%s" % line)
         self._hsp.identities = _safe_int(x), _safe_int(y)
+        self._hsp.align_length = _safe_int(y)
 
         if line.find('Positives') != -1:
             x, y = _re_search(
                 r"Positives = (\d+)\/(\d+)", line,
                 "I could not find the positives in line\n%s" % line)
             self._hsp.positives = _safe_int(x), _safe_int(y)
+            assert self._hsp.align_length == _safe_int(y)
 
         if line.find('Gaps') != -1:
             x, y = _re_search(
                 r"Gaps = (\d+)\/(\d+)", line,
                 "I could not find the gaps in line\n%s" % line)
             self._hsp.gaps = _safe_int(x), _safe_int(y)
+            assert self._hsp.align_length == _safe_int(y)
 
         
     def strand(self, line):
@@ -1114,7 +1121,7 @@ class _HSPConsumer:
     def query(self, line):
         m = self._query_re.search(line)
         if m is None:
-            raise ValueError, "I could not find the query in line\n%s" % line
+            raise ValueError("I could not find the query in line\n%s" % line)
         
         # line below modified by Yair Benita, Sep 2004.
         # added the end attribute for the query
@@ -1137,8 +1144,8 @@ class _HSPConsumer:
             # Make sure the alignment is the same length as the query
             seq = seq + ' ' * (self._query_len-len(seq))
         elif len(seq) < self._query_len:
-            raise ValueError, "Match is longer than the query in line\n%s" % \
-                  line
+            raise ValueError("Match is longer than the query in line\n%s" \
+                             % line)
         self._hsp.match = self._hsp.match + seq
 
     # To match how we do the query, cache the regular expression.
@@ -1147,7 +1154,7 @@ class _HSPConsumer:
     def sbjct(self, line):
         m = self._sbjct_re.search(line)
         if m is None:
-            raise ValueError, "I could not find the sbjct in line\n%s" % line
+            raise ValueError("I could not find the sbjct in line\n%s" % line)
         colon, start, seq, end = m.groups()
         #mikep 26/9/00
         #On occasion, there is a blast hit with no subject match
@@ -1161,9 +1168,9 @@ class _HSPConsumer:
 
         self._hsp.sbjct_end = _safe_int(end)
         if len(seq) != self._query_len:
-            raise ValueError, \
+            raise ValueError( \
                   "QUERY and SBJCT sequence lengths don't match in line\n%s" \
-                  % line
+                  % line)
 
         del self._query_start_index   # clean up unused variables
         del self._query_len
@@ -1409,8 +1416,7 @@ class _BlastConsumer(AbstractConsumer,
 
     def round(self, line):
         # Make sure nobody's trying to pass me PSI-BLAST data!
-        raise ValueError, \
-              "This consumer doesn't handle PSI-BLAST data"
+        raise ValueError("This consumer doesn't handle PSI-BLAST data")
         
     def start_header(self):
         self.data = Record.Blast()
@@ -1435,7 +1441,7 @@ class _BlastConsumer(AbstractConsumer,
         try:
             self._alignment.hsps.append(self._hsp)
         except AttributeError:
-            raise ValueError, "Found an HSP before an alignment"
+            raise ValueError("Found an HSP before an alignment")
 
     def end_database_report(self):
         _DatabaseReportConsumer.end_database_report(self)
@@ -1491,7 +1497,7 @@ class _PSIBlastConsumer(AbstractConsumer,
         try:
             self._alignment.hsps.append(self._hsp)
         except AttributeError:
-            raise ValueError, "Found an HSP before an alignment"
+            raise ValueError("Found an HSP before an alignment")
 
     def end_database_report(self):
         _DatabaseReportConsumer.end_database_report(self)
@@ -1557,8 +1563,7 @@ class Iterator:
         return iter(self.next, None)
 
 def blastall(blastcmd, program, database, infile, align_view='7', **keywds):
-    """blastall(blastcmd, program, database, infile, align_view='7', **keywds)
-    -> read, error Undohandles
+    """Execute and retrieve data from standalone BLASTPALL as handles.
     
     Execute and retrieve data from blastall.  blastcmd is the command
     used to launch the 'blastall' executable.  program is the blast program
@@ -1566,6 +1571,8 @@ def blastall(blastcmd, program, database, infile, align_view='7', **keywds):
     to search against.  infile is the path to the file containing
     the sequence to search with.
 
+    The return values are two handles, for standard output and standard error.
+
     You may pass more parameters to **keywds to change the behavior of
     the search.  Otherwise, optional values will be chosen by blastall.
     The Blast output is by default in XML format. Use the align_view keyword
@@ -1593,7 +1600,7 @@ def blastall(blastcmd, program, database, infile, align_view='7', **keywds):
     search_length       Effective length of search space.
 
         Processing
-    filter              Filter query sequence?  T/F
+    filter              Filter query sequence for low complexity (with SEG)?  T/F
     believe_query       Believe the query defline.  T/F
     restrict_gi         Restrict search to these GI's.
     nprocessors         Number of processors to use.
@@ -1609,6 +1616,9 @@ def blastall(blastcmd, program, database, infile, align_view='7', **keywds):
     seqalign_file       seqalign file to output.
 
     """
+
+    _security_check_parameters(keywds)
+
     att2param = {
         'matrix' : '-M',
         'gap_open' : '-G',
@@ -1646,33 +1656,27 @@ def blastall(blastcmd, program, database, infile, align_view='7', **keywds):
         'seqalign_file' : '-O'
         }
 
-    if not os.path.exists(blastcmd):
-        raise ValueError, "blastall does not exist at %s" % blastcmd
-    
     params = []
-
     params.extend([att2param['program'], program])
     params.extend([att2param['database'], database])
-    params.extend([att2param['infile'], infile])
+    params.extend([att2param['infile'], _escape_filename(infile)])
     params.extend([att2param['align_view'], str(align_view)])
 
     for attr in keywds.keys():
         params.extend([att2param[attr], str(keywds[attr])])
 
-    w, r, e = os.popen3(" ".join([blastcmd] + params))
-    w.close()
-    return File.UndoHandle(r), File.UndoHandle(e)
-
+    return _invoke_blast(blastcmd, params)
 
 def blastpgp(blastcmd, database, infile, align_view='7', **keywds):
-    """blastpgp(blastcmd, database, infile, align_view='7', **keywds) ->
-    read, error Undohandles
+    """Execute and retrieve data from standalone BLASTPGP as handles.
     
     Execute and retrieve data from blastpgp.  blastcmd is the command
     used to launch the 'blastpgp' executable.  database is the path to the
     database to search against.  infile is the path to the file containing
     the sequence to search with.
 
+    The return values are two handles, for standard output and standard error.
+
     You may pass more parameters to **keywds to change the behavior of
     the search.  Otherwise, optional values will be chosen by blastpgp.
     The Blast output is by default in XML format. Use the align_view keyword
@@ -1707,7 +1711,7 @@ def blastpgp(blastcmd, database, infile, align_view='7', **keywds):
         Processing
     XXX should document default values
     program             The blast program to use. (PHI-BLAST)
-    filter              Filter query sequence with SEG?  T/F
+    filter              Filter query sequence  for low complexity (with SEG)?  T/F
     believe_query       Believe the query defline?  T/F
     nprocessors         Number of processors to use.
 
@@ -1724,9 +1728,13 @@ def blastpgp(blastcmd, database, infile, align_view='7', **keywds):
     restart_infile      Input file for PSI-BLAST restart.
     hit_infile          Hit file for PHI-BLAST.
     matrix_outfile      Output file for PSI-BLAST matrix in ASCII.
+
     align_infile        Input alignment file for PSI-BLAST restart.
     
     """
+
+    _security_check_parameters(keywds)
+
     att2param = {
         'matrix' : '-M',
         'gap_open' : '-G',
@@ -1772,33 +1780,27 @@ def blastpgp(blastcmd, database, infile, align_view='7', **keywds):
         'matrix_outfile' : '-Q',
         'align_infile' : '-B'
         }
-        
-    if not os.path.exists(blastcmd):
-        raise ValueError, "blastpgp does not exist at %s" % blastcmd
-    
-    params = []
 
+    params = []
     params.extend([att2param['database'], database])
-    params.extend([att2param['infile'], infile])
+    params.extend([att2param['infile'], _escape_filename(infile)])
     params.extend([att2param['align_view'], str(align_view)])
 
     for attr in keywds.keys():
         params.extend([att2param[attr], str(keywds[attr])])
 
-    w, r, e = os.popen3(" ".join([blastcmd] + params))
-    w.close()
-    return File.UndoHandle(r), File.UndoHandle(e)
-
+    return _invoke_blast(blastcmd, params)
 
 def rpsblast(blastcmd, database, infile, align_view="7", **keywds):
-    """rpsblast(blastcmd, database, infile, **keywds) ->
-    read, error Undohandles
+    """Execute and retrieve data from standalone RPS-BLAST as handles.
     
     Execute and retrieve data from standalone RPS-BLAST.  blastcmd is the
     command used to launch the 'rpsblast' executable.  database is the path
     to the database to search against.  infile is the path to the file
     containing the sequence to search with.
 
+    The return values are two handles, for standard output and standard error.
+
     You may pass more parameters to **keywds to change the behavior of
     the search.  Otherwise, optional values will be chosen by rpsblast.
 
@@ -1828,7 +1830,7 @@ def rpsblast(blastcmd, database, infile, align_view="7", **keywds):
     db_length           Effective database length.
 
         Processing
-    filter              Filter query sequence with SEG?  T/F
+    filter              Filter query sequence for low complexity?  T/F
     case_filter         Use lower case filtering of FASTA sequence T/F, default F
     believe_query       Believe the query defline.  T/F
     nprocessors         Number of processors to use.
@@ -1845,6 +1847,9 @@ def rpsblast(blastcmd, database, infile, align_view="7", **keywds):
     align_outfile       Output file for alignment.
     
     """
+
+    _security_check_parameters(keywds)
+    
     att2param = {
         'multihit' : '-P',
         'gapped' : '-g',
@@ -1875,26 +1880,21 @@ def rpsblast(blastcmd, database, infile, align_view="7", **keywds):
         'align_outfile' : '-o'
         }
         
-    if not os.path.exists(blastcmd):
-        raise ValueError, "rpsblast does not exist at %s" % blastcmd
-    
     params = []
 
     params.extend([att2param['database'], database])
-    params.extend([att2param['infile'], infile])
+    params.extend([att2param['infile'], _escape_filename(infile)])
     params.extend([att2param['align_view'], str(align_view)])
 
     for attr in keywds.keys():
         params.extend([att2param[attr], str(keywds[attr])])
 
-    w, r, e = os.popen3(" ".join([blastcmd] + params))
-    w.close()
-    return File.UndoHandle(r), File.UndoHandle(e)
+    return _invoke_blast(blastcmd, params)
 
 def _re_search(regex, line, error_msg):
     m = re.search(regex, line)
     if not m:
-        raise ValueError, error_msg
+        raise ValueError(error_msg)
     return m.groups()
 
 def _get_cols(line, cols_to_get, ncols=None, expected={}):
@@ -1902,14 +1902,14 @@ def _get_cols(line, cols_to_get, ncols=None, expected={}):
 
     # Check to make sure number of columns is correct
     if ncols is not None and len(cols) != ncols:
-        raise ValueError, "I expected %d columns (got %d) in line\n%s" % \
-              (ncols, len(cols), line)
+        raise ValueError("I expected %d columns (got %d) in line\n%s" \
+                         % (ncols, len(cols), line))
 
     # Check to make sure columns contain the correct data
     for k in expected.keys():
         if cols[k] != expected[k]:
-            raise ValueError, "I expected '%s' in column %d in line\n%s" % (
-                expected[k], k, line)
+            raise ValueError("I expected '%s' in column %d in line\n%s" \
+                             % (expected[k], k, line))
 
     # Construct the answer tuple
     results = []
@@ -1949,12 +1949,77 @@ def _safe_float(str):
     # try again.
     return float(str)
 
+def _escape_filename(filename) :
+    """Escape filenames with spaces (PRIVATE)."""
+    if " " not in filename :
+        return filename
+
+    #See Bug 2480 - is adding the following helpful?
+    #if os.path.isfile(filename) :
+    #    #On Windows, if the file exists, we can ask for
+    #    #its alternative short name (DOS style 8.3 format)
+    #    #which has no spaces in it.
+    #    try :
+    #        import win32api
+    #        short = win32api.GetShortPathName(filename)
+    #        assert os.path.isfile(short)
+    #        return short
+    #    except ImportError :
+    #        pass
+
+    #We'll just quote it - works on Mac etc
+    if filename.startswith('"') and filename.endswith('"') :
+        #Its already quoted
+        return filename
+    else :
+        return '"%s"' % filename
+
+def _invoke_blast(blast_cmd, params) :
+    """Start BLAST and returns handles for stdout and stderr (PRIVATE).
+
+    Tries to deal with spaces in the BLAST executable path.
+    """
+    if not os.path.exists(blast_cmd):
+        raise ValueError("BLAST executable does not exist at %s" % blast_cmd)
+
+    cmd_string = " ".join([_escape_filename(blast_cmd)] + params)
+
+    try :
+        import subprocess, sys
+        blast_process = subprocess.Popen(cmd_string,
+                                         stdout=subprocess.PIPE,
+                                         stderr=subprocess.PIPE,
+                                         shell=(sys.platform!="win32"))
+        return blast_process.stdout, blast_process.stderr
+    except ImportError :
+        #subprocess isn't available on python 2.3
+        #Note os.popen3 is deprecated in python 2.6
+        write_handle, result_handle, error_handle \
+                      = os.popen3(cmd_string)
+        write_handle.close()
+        return result_handle, error_handle
+
+def _security_check_parameters(param_dict) :
+    """Look for any attempt to insert a command into a parameter.
+
+    e.g. blastall(..., matrix='IDENTITY -F 0; rm -rf /etc/passwd')
+
+    Looks for ";" or "&&" in the strings (Unix and Windows syntax
+    for appending a command line), or ">", "<" or "|" (redirection)
+    and if any are found raises an exception.
+    """
+    for key, value in param_dict.iteritems() :
+        str_value = str(value) # Could easily be an int or a float
+        for bad_str in [";", "&&", ">", "<", "|"] :
+            if bad_str in str_value :
+                raise ValueError("Rejecting suspicious argument for %s" % key)
+
 class _BlastErrorConsumer(_BlastConsumer):
     def __init__(self):
         _BlastConsumer.__init__(self)
     def noevent(self, line):
         if line.find("Query must be at least wordsize") != -1:
-            raise ShortQueryBlastError, "Query must be at least wordsize"
+            raise ShortQueryBlastError("Query must be at least wordsize")
         # Now pass the line back up to the superclass.
         method = getattr(_BlastConsumer, 'noevent',
                          _BlastConsumer.__getattr__(self, 'noevent'))
