@@ -9,15 +9,16 @@
 """
 This module provides code to work with the WWW version of BLAST
 provided by the NCBI.
-http://www.ncbi.nlm.nih.gov/BLAST/
-
-Classes:
-BlastParser   Parses output from WWW blast.
-_Scanner      Scans output from NCBI's BLAST WWW server.
+http://blast.ncbi.nlm.nih.gov/
 
 Functions:
 qblast        Do a BLAST search using the QBLAST API.
 
+Deprecated classes:
+BlastParser   Parses output from WWW blast.
+_Scanner      Scans output from NCBI's BLAST WWW server.
+
+
 """
 import re
 
@@ -29,11 +30,18 @@ except ImportError:
 from Bio.ParserSupport import *
 
 class BlastParser(AbstractParser):
-    """Parses WWW BLAST data into a Record.Blast object.
+    """Parses WWW BLAST data into a Record.Blast object (DEPRECATED).
 
+    This is a parser for the NCBI's HTML (web page) BLAST output.
     """
     def __init__(self):
-        """__init__(self)"""
+        """Create a BlastParser object (DEPRECATED)."""
+        import warnings      
+        warnings.warn("Bio.Blast.NCBIWWW.BlastParser is deprecated." \
+                      + " We recommend you use the XML output with" \
+                      + " the parser in Bio.Blast.NCBIXML instead.",
+                      DeprecationWarning)
+                       
         import NCBIStandalone
         self._scanner = _Scanner()
         self._consumer = SGMLStrippingConsumer(NCBIStandalone._BlastConsumer())
@@ -44,14 +52,15 @@ class BlastParser(AbstractParser):
         return self._consumer.data
     
 class _Scanner:
-    """Scan BLAST output from NCBI's web server at:
+    """Scanner for the HTML BLAST parser (PRIVATE, DEPRECATED).
+    
+    Scan BLAST output from NCBI's web server at:
     http://www.ncbi.nlm.nih.gov/BLAST/
     
     Tested with BLAST v2.0.10
 
     Methods:
     feed     Feed data into the scanner.
-    
     """
     def feed(self, handle, consumer):
         """S.feed(handle, consumer)
@@ -94,7 +103,7 @@ class _Scanner:
                             has_re=re.compile(r'<b>.?BLAST'))
 
         self._scan_header(uhandle, consumer)
-	self._scan_rounds(uhandle, consumer)
+        self._scan_rounds(uhandle, consumer)
         self._scan_database_report(uhandle, consumer)
         self._scan_parameters(uhandle, consumer)
 
@@ -323,7 +332,8 @@ class _Scanner:
         elif line1.startswith('<PRE>'):
             is_pairwise = 1
         else:
-            raise ValueError, "Cannot resolve location at lines:\n%s\n%s" % (line1, line2)
+            raise ValueError("Cannot resolve location at lines:\n%s\n%s" \
+                             % (line1, line2))
 
         if is_pairwise:
             self._scan_pairwise_alignments(uhandle, consumer)
@@ -420,7 +430,7 @@ class _Scanner:
                 break
             elif is_blank_line(line):
                 # Check to make sure I haven't missed the Length line
-                raise ValueError, "I missed the Length in an alignment header"
+                raise ValueError("I missed the Length in an alignment header")
             consumer.title(line)
 
         if not attempt_read_and_call(uhandle, consumer.noevent,
@@ -684,8 +694,8 @@ def qblast(program, database, sequence,
 
     Supports all parameters of the qblast API for Put and Get.
     Some useful parameters:
-    program        BLASTP or BLASTN
-    database       Which database to search against.
+    program        blastn, blastp, blastx, tblastn, or tblastx (lower case)
+    database       Which database to search against (e.g. "nr").
     sequence       The sequence to search.
     ncbi_gi        TRUE/FALSE whether to give 'gi' identifier.
     descriptions   Number of descriptions to show.  Def 500.
@@ -703,9 +713,9 @@ def qblast(program, database, sequence,
 
     """
     import urllib, urllib2
-    from Bio.WWW import RequestLimiter
+    import time
 
-    assert program == 'blastn' or program == 'blastp'
+    assert program in ['blastn', 'blastp', 'blastx', 'tblastn', 'tblastx']
 
     # Format the "Put" command, which sends search requests to qblast.
     # Parameters taken from http://www.ncbi.nlm.nih.gov/BLAST/Doc/node5.html on 9 July 2007
@@ -747,7 +757,10 @@ def qblast(program, database, sequence,
     message = urllib.urlencode(query)
 
     # Send off the initial query to qblast.
-    request = urllib2.Request("http://www.ncbi.nlm.nih.gov/blast/Blast.cgi",
+    # Note the NCBI do not currently impose a rate limit here, other
+    # than the request not to make say 50 queries at once using multiple
+    # threads.
+    request = urllib2.Request("http://blast.ncbi.nlm.nih.gov/Blast.cgi",
                               message,
                               {"User-Agent":"BiopythonClient"})
     handle = urllib2.urlopen(request)
@@ -775,11 +788,19 @@ def qblast(program, database, sequence,
     query = [x for x in parameters if x[1] is not None]
     message = urllib.urlencode(query)
 
-    # Poll NCBI until the results are ready.
-    limiter = RequestLimiter(3)
-    while 1:
-        limiter.wait()
-        request = urllib2.Request("http://www.ncbi.nlm.nih.gov/blast/Blast.cgi",
+    # Poll NCBI until the results are ready.  Use a 3 second wait
+    delay = 3.0
+    previous = time.time()
+    while True:
+        current = time.time()
+        wait = previous + delay - current
+        if wait > 0:
+            time.sleep(wait)
+            previous = current + wait
+        else:
+            previous = current
+
+        request = urllib2.Request("http://blast.ncbi.nlm.nih.gov/Blast.cgi",
                                   message,
                                   {"User-Agent":"BiopythonClient"})
         handle = urllib2.urlopen(request)
@@ -796,14 +817,22 @@ def qblast(program, database, sequence,
     return StringIO.StringIO(results)
 
 def _parse_qblast_ref_page(handle):
-    """Return tuple of RID, RTOE."""
+    """Extract a tuple of RID, RTOE from the 'please wait' page (PRIVATE).
+    """
     s = handle.read()
     i = s.find("RID =")
+    if i == 1 :
+        raise ValueError("No RID found in the 'please wait' page.")
     j = s.find("\n", i)
     rid = s[i+len("RID ="):j].strip()
 
     i = s.find("RTOE =")
+    if i == 1 :
+        raise ValueError("No RTOE found in the 'please wait' page.")
     j = s.find("\n", i)
     rtoe = s[i+len("RTOE ="):j].strip()
-    return rid, int(rtoe)
-    
+    try :
+        return rid, int(rtoe)
+    except ValueError :
+        raise ValueError("A non-integer RTOE found in " \
+                         +"the 'please wait' page, %s" % repr(rtoe))
