@@ -1,6 +1,6 @@
 # Copyright 2000 by Bertrand Frottier .  All rights reserved.
 # Revisions 2005-2006 copyright Michiel de Hoon
-# Revisions 2006 copyright Peter Cock
+# Revisions 2006-2008 copyright Peter Cock
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -189,6 +189,8 @@ class BlastParser(_XMLparser):
         self._blast.gap_penalties = self._parameters.gap_penalties
         self._blast.filter = self._parameters.filter
         self._blast.expect = self._parameters.expect
+        self._blast.sc_match = self._parameters.sc_match
+        self._blast.sc_mismatch = self._parameters.sc_mismatch
 
         #Add to the list
         self._records.append(self._blast)
@@ -206,12 +208,27 @@ class BlastParser(_XMLparser):
         self._header.application = self._value.upper()
 
     def _end_BlastOutput_version(self):
-        """version number of the BLAST engine (e.g., 2.1.2)
+        """version number and date of the BLAST engine.
+
+        e.g. "BLASTX 2.2.12 [Aug-07-2005]" but there can also be
+        variants like "BLASTP 2.2.18+" without the date.
 
         Save this to put on each blast record object
         """
-        self._header.version = self._value.split()[1]
-        self._header.date = self._value.split()[2][1:-1]
+        parts = self._value.split()
+        #TODO - Check the first word starts with BLAST?
+
+        #The version is the second word (field one)
+        self._header.version = parts[1]
+        
+        #Check there is a third word (the date)
+        if len(parts) >= 3 :
+            if parts[2][0] == "[" and parts[2][-1] == "]" :
+                self._header.date = parts[2][1:-1]
+            else :
+                #Assume this is still a date, but without the
+                #square brackets
+                self._header.date = parts[2]
 
     def _end_BlastOutput_reference(self):
         """a reference to the article describing the algorithm
@@ -404,8 +421,8 @@ class BlastParser(_XMLparser):
         """bit score of HSP
         """
         self._hsp.bits = float(self._value)
-        #if self._descr.bits == None:
-        #    self._descr.bits = float(self._value)
+        if self._descr.bits == None:
+            self._descr.bits = float(self._value)
 
     def _end_Hsp_evalue(self):
         """expect value value of the HSP
@@ -503,7 +520,7 @@ class BlastParser(_XMLparser):
     def _end_Statistics_db_len(self):
         """number of letters in the database
         """
-        self._blast._num_letters_in_database = int(self._value)
+        self._blast.num_letters_in_database = int(self._value)
 
     def _end_Statistics_hsp_len(self):
         """the effective HSP length
@@ -563,7 +580,8 @@ def parse(handle, debug=0):
     while text :
         #We are now starting a new XML file
         if not text.startswith(XML_START) :
-            raise ValueError("Your XML file did not start with <?xml...")
+            raise ValueError("Your XML file did not start with %s..." \
+                             % XML_START)
 
         expat_parser = expat.ParserCreate()
         blast_parser = BlastParser(debug)
