@@ -12,31 +12,23 @@ This requires clustalw available from:
 ftp://ftp-igbmc.u-strasbg.fr/pub/ClustalW/.
 
 functions:
+o read
 o parse_file
 o do_alignment
 
 classes:
 o ClustalAlignment
-o _AlignCreator
 o MultipleAlignCL"""
 
 # standard library
 import os
 import sys
-import string #Obsolete - we should switch to using string object methods instead!
 
 # biopython
-from Bio.Seq import Seq
-from Bio.SeqRecord import SeqRecord
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
-import clustal_format
 from Bio.Align.Generic import Alignment
 
-# PyXML package
-from xml.sax import saxutils
-from xml.sax import handler
-
 def parse_file(file_name, alphabet = IUPAC.unambiguous_dna, debug_level = 0):
     """Parse the given file into a clustal aligment object.
     
@@ -45,22 +37,45 @@ def parse_file(file_name, alphabet = IUPAC.unambiguous_dna, debug_level = 0):
     o alphabet - The type of alphabet to use for the alignment sequences.
     This should correspond to the type of information contained in the file.
     Defaults to be unambiguous_dna sequence.
+
+    There is a deprecated optional argument debug_level which has no effect.
+
+    Since Biopython 1.46, this has called Bio.AlignIO internally.
     """ 
-    align_handler = _AlignCreator(Alphabet.Gapped(alphabet))
 
-    parser = clustal_format.format.make_parser(debug_level)
-    parser.setContentHandler(align_handler)
-    parser.setErrorHandler(handler.ErrorHandler())
+    # Avoid code duplication by calling Bio.AlignIO to do this for us.
+    handle = open(file_name, 'r')
+    from Bio import AlignIO
+    generic_alignment = AlignIO.read(handle, "clustal")
+    handle.close()
 
-    to_parse = open(file_name, 'r')
-    parser.parseFile(to_parse)
-    to_parse.close()
+    #Force this generic alignment into a ClustalAlignment... nasty hack
+    if isinstance(alphabet, Alphabet.Gapped) :
+        alpha = alphabet
+    else :
+        alpha = Alphabet.Gapped(alphabet)
+    clustal_alignment = ClustalAlignment(alpha)
+    clustal_alignment._records = generic_alignment._records
+    for record in clustal_alignment._records :
+        record.seq.alphabet = alpha
 
-    return align_handler.align
+    try :
+        clustal_alignment._version = generic_alignment._version
+    except AttributeError :
+        #Missing the version, could be a 3rd party tool's output
+        pass
+
+    try :       
+        clustal_alignment._star_info = generic_alignment._star_info
+    except AttributeError :
+        #Missing the consensus, again, this is not always present
+        pass
+
+    return clustal_alignment
 
 def do_alignment(command_line, alphabet=None):
     """Perform an alignment with the given command line.
-
+    
     Arguments:
     o command_line - A command line object that can give out
     the command line we will input into clustalw.
@@ -72,8 +87,18 @@ def do_alignment(command_line, alphabet=None):
     o A clustal alignment object corresponding to the created alignment.
     If the alignment type was not a clustal object, None is returned.
     """
-    run_clust = os.popen(str(command_line))
-    status = run_clust.close()
+    try :
+        import subprocess
+        child_process = subprocess.Popen(str(command_line),
+                                         stdout=subprocess.PIPE,
+                                         stderr=subprocess.PIPE,
+                                         shell=(sys.platform!="win32")
+                                         )
+        status = child_process.wait()
+    except ImportError :
+        #Fall back for python 2.3
+        run_clust = os.popen(str(command_line))
+        status = run_clust.close()
 
     
     # The exit status is the second byte of the termination status
@@ -145,160 +170,30 @@ class ClustalAlignment(Alignment):
         The output produced from this should also be formatted in valid
         clustal format.
         """
-        # if the version isn't set, we need to use the default
-        if self._version == '':
-            self._version = self.DEFAULT_VERSION
-        
-        output = "CLUSTAL X (%s) multiple sequence alignment\n\n\n" % \
-                 self._version
-
-        cur_char = 0
-        max_length = len(self._records[0].seq)
-
-        # keep displaying sequences until we reach the end
-        while cur_char != max_length:
-            # calculate the number of sequences to show, which will
-            # be less if we are at the end of the sequence
-            if (cur_char + 50) > max_length:
-                show_num = max_length - cur_char
-            else:
-                show_num = 50
-
-            # go through all of the records and print out the sequences
-            # when we output, we do a nice 80 column output, although this
-            # may result in truncation of the ids.
-            for record in self._records:
-                line = record.description[0:30].ljust(36)
-                line = line + record.seq.data[cur_char:(cur_char + show_num)]
-                
-                output = output + line + "\n"
-
-            # now we need to print out the star info, if we've got it
-            if self._star_info != '':
-                output = output + (" " * 36) + \
-                     self._star_info[cur_char:(cur_char + show_num)] + "\n"
-
-            output = output + "\n"
-            cur_char = cur_char + show_num
-
-        # have a extra newline, so strip two off and add one before returning
-        return string.rstrip(output) + "\n"
+        # Avoid code duplication by calling Bio.AlignIO to do this for us.
+        from Bio import AlignIO
+        from StringIO import StringIO
+        handle = StringIO()
+        AlignIO.write([self], handle, "clustal")
+        handle.seek(0)
+        return handle.read()
             
+def _escape_filename(filename) :
+    """Escape filenames with spaces (PRIVATE).
 
-    def _add_star_info(self, stars):
-        """Add all of the stars, which indicate consensus sequence.
-        """
-        self._star_info = stars
-
-    def _add_version(self, version):
-        """Add the version information about the clustal file being read.
-        """
-        self._version = version
-
-class _AlignCreator(handler.ContentHandler):
-    """Handler to create a ClustalAlignment object from clustal file info.
-
-    This handler is used to accept events coming from a Martel parsing
-    stream, and acts like a normal SAX handler.
-
-    After parsing, the alignment object created is available as the
-    align attribute of the class.
+    NOTE - This code is duplicated in Bio.Blast.NCBIStandalone,
+    scope for refactoring!
     """
-    def __init__(self, alphabet):
-        """Create a new handler ready to deal with output from Martel parsing.
+    if " " not in filename :
+        return filename
 
-        Arguments:
-        o alphabet - The alphabet to create all of the new sequences with.
-        """
-        self.align = ClustalAlignment(alphabet)
+    #We'll just quote it - works on Mac etc
+    if filename.startswith('"') and filename.endswith('"') :
+        #Its already quoted
+        return filename
+    else :
+        return '"%s"' % filename
 
-        # store sequence info in a dictionary
-        self.all_info = {}
-        self.all_keys = []
-
-        # the current id we are working with
-        self.cur_id = None
-
-        # info so we know how big the ids and sequences are
-        self.id_size = 0
-        self.space_size = 0
-        self.seq_size = 0
-        
-        # flags so we can keep track of where we are during the parse
-        self.in_version = 0
-        self.in_stars = 0
-        self.in_seq_id = 0
-        self.in_space = 0
-        self.in_seq = 0
-        self.all_star_info = ''
-
-    def startElement(self, name, attrs):
-        """Check the various tags for the info we are interested in."""
-        if name == "version":
-            self.in_version = 1
-            self.version_info = ''
-        elif name == "seq_id":
-            self.in_seq_id = 1
-            self.seq_id_info = ''
-        elif name == "seq_space":
-            self.in_space = 1
-            self.space_info = ''
-        elif name == "seq_info":
-            self.in_seq = 1
-            self.seq_info = ''
-        elif name == "match_stars":
-            self.in_stars = 1
-            self.star_info = ''
-
-    def characters(self, content):
-        if self.in_version:
-            self.version_info = self.version_info + content
-        elif self.in_seq_id:
-            self.seq_id_info = self.seq_id_info + content
-        elif self.in_space:
-            self.space_info = self.space_info + content
-        elif self.in_seq:
-            self.seq_info = self.seq_info + content
-        elif self.in_stars:
-            self.star_info = self.star_info + content
-
-    def endElement(self, name):
-        if name == "version":
-            self.in_version = 0
-            self.align._add_version(string.strip(self.version_info))
-        elif name == "seq_id":
-            self.in_seq_id = 0
-            self.id_size = len(self.seq_id_info)
-            self.cur_id = self.seq_id_info
-        elif name == "seq_space":
-            self.in_space = 0
-            self.space_size = len(self.space_info)
-        elif name == "seq_info":
-            self.in_seq = 0
-            self.seq_size = len(self.seq_info)
-
-            # if the id is already there, add the sequence info
-            if self.cur_id in self.all_info.keys():
-                self.all_info[self.cur_id] = self.all_info[self.cur_id] + \
-                                             self.seq_info
-            else:
-                self.all_info[self.cur_id] = self.seq_info
-                self.all_keys.append(self.cur_id)
-                
-        elif name == "match_stars":
-            id_length = self.id_size + self.space_size
-            line_length = id_length + self.seq_size
-            
-            self.all_star_info = self.all_star_info + \
-                            self.star_info[id_length:line_length]
-                                                                           
-    def endDocument(self):
-        # when we are done parsing add all of the info we need
-        self.align._add_star_info(self.all_star_info)
-
-        for id in self.all_keys:
-            self.align.add_sequence(id, self.all_info[id])
-        
 class MultipleAlignCL:
     """Represent a clustalw multiple alignment command line.
 
@@ -384,60 +279,51 @@ class MultipleAlignCL:
     def __str__(self):
         """Write out the command line as a string."""
 
-        if sys.platform <> "win32" :
-            #On Linux with clustalw 1.83, you can do:
-            #clustalw input.faa
-            #clustalw /full/path/input.faa
-            #clustalw -INFILE=input.faa
-            #clustalw -INFILE=/full/path/input.faa
-            #
-            #Note these fail (using DOS style slashes):
-            #
-            #clustalw /INFILE=input.faa
-            #clustalw /INFILE=/full/path/input.faa
-            #
-            #To keep things simple, and follow the original
-            #behaviour of Bio.Clustalw use this:
-            cline = self.command + " " + self.sequence_file
-        else :
-            #On Windows XP with clustalw.exe 1.83, these work at
-            #the command prompt:
-            #
-            #clustalw.exe input.faa
-            #clustalw.exe /INFILE=input.faa
-            #clustalw.exe /INFILE="input.faa"
-            #clustalw.exe /INFILE="with space.faa"
-            #clustalw.exe /INFILE=C:\full\path\input.faa
-            #clustalw.exe /INFILE="C:\full path\with spaces.faa"
-            #
-            #Sadly these fail:
-            #clustalw.exe "input.faa"
-            #clustalw.exe "with space.faa"
-            #clustalw.exe C:\full\path\input.faa
-            #clustalw.exe "C:\full path\with spaces.faa"
-            #
-            #These also fail but a minus/dash does seem to
-            #work with other options (!):
-            #clustalw.exe -INFILE=input.faa
-            #clustalw.exe -INFILE=C:\full\path\input.faa
-            #
-            #Also these fail:
-            #clustalw.exe "/INFILE=input.faa"
-            #clustalw.exe "/INFILE=C:\full\path\input.faa"
-            #
-            #Thanks to Emanuel Hey for flagging this on the mailing list.
-            #
-            #In addtion, both self.command and self.sequence_file
-            #may contain spaces, so should be quoted. But clustalw
-            #is fussy.
-            if self.command.count(" ") > 0 :
-                cline = '"%s"' % self.command
-            else :
-                cline = self.command
-            if self.sequence_file.count(" ") > 0 :
-                cline += ' /INFILE="%s"' % self.sequence_file
-            else :
-                cline += ' /INFILE=%s' % self.sequence_file
+        #On Linux with clustalw 1.83, you can do:
+        #clustalw input.faa
+        #clustalw /full/path/input.faa
+        #clustalw -INFILE=input.faa
+        #clustalw -INFILE=/full/path/input.faa
+        #
+        #Note these fail (using DOS style slashes):
+        #
+        #clustalw /INFILE=input.faa
+        #clustalw /INFILE=/full/path/input.faa
+        #
+        #On Windows XP with clustalw.exe 1.83, these work at
+        #the command prompt:
+        #
+        #clustalw.exe input.faa
+        #clustalw.exe /INFILE=input.faa
+        #clustalw.exe /INFILE="input.faa"
+        #clustalw.exe /INFILE="with space.faa"
+        #clustalw.exe /INFILE=C:\full\path\input.faa
+        #clustalw.exe /INFILE="C:\full path\with spaces.faa"
+        #
+        #Sadly these fail:
+        #clustalw.exe "input.faa"
+        #clustalw.exe "with space.faa"
+        #clustalw.exe C:\full\path\input.faa
+        #clustalw.exe "C:\full path\with spaces.faa"
+        #
+        #Testing today (using a different binary of clustalw.exe 1.83),
+        #using -INFILE as follows seems to work.  However I had once noted:
+        #These also fail but a minus/dash does seem to
+        #work with other options (!):
+        #clustalw.exe -INFILE=input.faa
+        #clustalw.exe -INFILE=C:\full\path\input.faa
+        #
+        #Also these fail:
+        #clustalw.exe "/INFILE=input.faa"
+        #clustalw.exe "/INFILE=C:\full\path\input.faa"
+        #
+        #Thanks to Emanuel Hey for flagging this on the mailing list.
+        #
+        #In addition, both self.command and self.sequence_file
+        #may contain spaces, so should be quoted. But clustalw
+        #is fussy.
+        cline = _escape_filename(self.command)
+        cline += ' -INFILE=%s' % _escape_filename(self.sequence_file)
 
         # general options
         if self.type:
@@ -451,7 +337,7 @@ class MultipleAlignCL:
 
         # output options
         if self.output_file:
-            cline += " -OUTFILE=%s" % self.output_file
+            cline += " -OUTFILE=%s" % _escape_filename(self.output_file)
         if self.output_type:
             cline += " -OUTPUT=%s" % self.output_type
         if self.output_order:
@@ -462,11 +348,11 @@ class MultipleAlignCL:
             cline += " -SEQNOS=%s" % self.add_seqnos
         if self.new_tree:
             # clustal does not work if -align is written -ALIGN
-            cline += " -NEWTREE=%s -align" % self.new_tree
+            cline += " -NEWTREE=%s -align" % _escape_filename(self.new_tree)
 
         # multiple alignment options
         if self.guide_tree:
-            cline += " -USETREE=%s" % self.guide_tree
+            cline += " -USETREE=%s" % _escape_filename(self.guide_tree)
         if self.protein_matrix:
             cline += " -MATRIX=%s" % self.protein_matrix
         if self.dna_matrix:
@@ -503,7 +389,7 @@ class MultipleAlignCL:
         self.output_file = output_file
 
         if output_type:
-            output_type = string.upper(output_type)
+            output_type = output_type.upper()
             if output_type not in self.OUTPUT_TYPES:
                 raise ValueError("Invalid output type %s. Valid choices are %s"
                                  % (output_type, self.OUTPUT_TYPES))
@@ -511,7 +397,7 @@ class MultipleAlignCL:
                 self.output_type = output_type
 
         if output_order:
-            output_order = string.upper(output_order)
+            output_order = output_order.upper()
             if output_order not in self.OUTPUT_ORDER:
                 raise ValueError("Invalid output order %s. Valid choices are %s"
                                  % (output_order, self.OUTPUT_ORDER))
@@ -519,7 +405,7 @@ class MultipleAlignCL:
                 self.output_order = output_order
 
         if change_case:
-            change_case = string.upper(change_case)
+            change_case = change_case.upper()
             if output_type != "GDE":
                 raise ValueError("Change case only valid for GDE output.")
             elif change_case not in self.CHANGE_CASE:
@@ -529,7 +415,7 @@ class MultipleAlignCL:
                 self.change_case = change_case
 
         if add_seqnos:
-            add_seqnos = string.upper(add_seqnos)
+            add_seqnos = add_seqnos.upper()
             if output_type:
                 raise ValueError("Add SeqNos only valid for CLUSTAL output.")
             elif add_seqnos not in self.OUTPUT_SEQNOS:
@@ -560,22 +446,21 @@ class MultipleAlignCL:
         Protein matrix can be either one of the defined types (blosum, pam,
         gonnet or id) or a file with your own defined matrix.
         """
-        if string.upper(protein_matrix) in self.PROTEIN_MATRIX:
-            self.protein_matrix = string.upper(protein_matrix)
+        if protein_matrix.upper() in self.PROTEIN_MATRIX:
+            self.protein_matrix = protein_matrix.upper()
         elif os.path.exists(protein_matrix):
             self.protein_matrix = protein_matrix
         else:
             raise ValueError("Invalid matrix %s. Options are %s or a file." %
-                             (string.upper(protein_matrix),
-                              self.PROTEIN_MATRIX))
+                             (protein_matrix.upper(), self.PROTEIN_MATRIX))
 
     def set_dna_matrix(self, dna_matrix):
         """Set the type of DNA matrix to use.
 
         The dna_matrix can either be one of the defined types (iub or clustalw)
         or a file with the matrix to use."""
-        if string.upper(dna_matrix) in self.DNA_MATRIX:
-            self.dna_matrix = string.upper(dna_matrix)
+        if dna_matrix.upper() in self.DNA_MATRIX:
+            self.dna_matrix = dna_matrix.upper()
         elif os.path.exists(dna_matrix):
             self.dna_matrix = dna_matrix
         else:
@@ -590,7 +475,7 @@ class MultipleAlignCL:
         protein or DNA you are working with, so this allows you to set it
         explicitly.
         """
-        residue_type = string.upper(residue_type)
+        residue_type = residue_type.upper()
         if residue_type in self.RESIDUE_TYPES:
             self.type = residue_type
         else:
