@@ -1,3 +1,9 @@
+#TODO - Remove this work around once we drop python 2.3 support
+try:
+   set = set
+except NameError:
+   from sets import Set as set
+
 import string
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
@@ -67,7 +73,7 @@ class CodonTable:
         else :
             #Should be either RNA or generic nucleotides,
             #e.g. Bio.Data.CodonTable.generic_by_id[1]
-            letters = "TCAG"
+            letters = "UCAG"
 
         #Build the table...
         answer=answer + "\n\n  |" + "|".join( \
@@ -256,7 +262,7 @@ def register_ncbi_table(name, alt_name, id,
 ##         line == 'Genetic-code-table ::= {\n':
 ##        pass
 ##    else:
-##        raise "Unparsed", repr(line)
+##        raise Exception("Unparsed: " + repr(line))
 
 register_ncbi_table(name = 'Standard',
                     alt_name = 'SGC0', id = 1,
@@ -527,8 +533,8 @@ class AmbiguousCodonTable(CodonTable):
                             # These two are WRONG!  I need to get the
                             # list of ambiguous codons which code for
                             # the stop codons  XXX
-                            list_ambiguous_codons(codon_table.start_codons),
-                            list_ambiguous_codons(codon_table.stop_codons)
+                            list_ambiguous_codons(codon_table.start_codons, ambiguous_nucleotide_values),
+                            list_ambiguous_codons(codon_table.stop_codons, ambiguous_nucleotide_values)
                             )
         self._codon_table = codon_table
 
@@ -555,14 +561,64 @@ def list_possible_proteins(codon, forward_table, ambiguous_nucleotide_values):
                         stops.append(y1+y2+y3)
         if stops:
             if possible.keys():
-                raise TranslationError, ("ambiguous codon codes for both proteins and stop codons", codon)
+                raise TranslationError("ambiguous codon '%s' codes " % codon \
+                                       + "for both proteins and stop codons")
             # This is a true stop codon - tell the caller about it
-            raise KeyError, codon
+            raise KeyError(codon)
         return possible.keys()
 
-def list_ambiguous_codons(codons):
-    # XXX not implemented!
-    return codons
+def list_ambiguous_codons(codons, ambiguous_nucleotide_values):
+    """Extends a codon list to include all possible ambigous codons.
+
+    e.g. ['TAG', 'TAA'] -> ['TAG', 'TAA', 'TAR']
+         ['UAG', 'UGA'] -> ['UAG', 'UGA', 'URA']
+
+    Note that ['TAG', 'TGA'] -> ['TAG', 'TGA'], this does not add 'TRR'.
+    Thus only two more codons are added in the following:
+
+    e.g. ['TGA', 'TAA', 'TAG'] -> ['TGA', 'TAA', 'TAG', 'TRA', 'TAR']
+
+    Returns a new (longer) list of codon strings.
+    """
+
+    #Note ambiguous_nucleotide_values['R'] = 'AG' (etc)
+    #This will generate things like 'TRR' from ['TAG', 'TGA'], which
+    #we don't want to include:
+    c1_list = [letter for (letter, meanings) \
+               in ambiguous_nucleotide_values.iteritems() \
+               if set([codon[0] for codon in codons]).issuperset(set(meanings))]
+    c2_list = [letter for (letter, meanings) \
+               in ambiguous_nucleotide_values.iteritems() \
+               if set([codon[1] for codon in codons]).issuperset(set(meanings))]
+    c3_list = [letter for (letter, meanings) \
+               in ambiguous_nucleotide_values.iteritems() \
+               if set([codon[2] for codon in codons]).issuperset(set(meanings))]
+    set2 = set([codon[1] for codon in codons])
+    set3 = set([codon[2] for codon in codons])
+    candidates = set([c1+c2+c3 for c1 in c1_list for c2 in c2_list for c3 in c3_list])
+    candidates.difference_update(codons)
+    answer = codons[:] #copy
+    #print "Have %i new candidates" % len(candidates)
+    for ambig_codon in candidates :
+        wanted = True
+        #e.g. 'TRR' -> 'TAA', 'TAG', 'TGA', 'TGG'
+        for codon in [c1+c2+c3 \
+                      for c1 in ambiguous_nucleotide_values[ambig_codon[0]] \
+                      for c2 in ambiguous_nucleotide_values[ambig_codon[1]] \
+                      for c3 in ambiguous_nucleotide_values[ambig_codon[2]]]:
+            if codon not in codons :
+                #This ambiguous codon can code for a non-stop, exclude it!
+                wanted=False
+                #print "Rejecting %s" % ambig_codon
+                continue
+        if wanted :
+            answer.append(ambig_codon)
+    return answer
+assert list_ambiguous_codons(['TGA', 'TAA'],IUPACData.ambiguous_dna_values) == ['TGA', 'TAA', 'TRA']
+assert list_ambiguous_codons(['TAG', 'TGA'],IUPACData.ambiguous_dna_values) == ['TAG', 'TGA']
+assert list_ambiguous_codons(['TAG', 'TAA'],IUPACData.ambiguous_dna_values) == ['TAG', 'TAA', 'TAR']
+assert list_ambiguous_codons(['UAG', 'UAA'],IUPACData.ambiguous_rna_values) == ['UAG', 'UAA', 'UAR']
+assert list_ambiguous_codons(['TGA', 'TAA', 'TAG'],IUPACData.ambiguous_dna_values) == ['TGA', 'TAA', 'TAG', 'TAR', 'TRA']
 
 # Forward translation is "onto", that is, any given codon always maps
 # to the same protein, or it doesn't map at all.  Thus, I can build
@@ -611,9 +667,9 @@ class AmbiguousForwardTable:
             pass
         else:
             if x is TranslationError:
-                raise TranslationError, codon   # no unique translation
+                raise TranslationError(codon)   # no unique translation
             if x is KeyError:
-                raise KeyError, codon  # it's a stop codon
+                raise KeyError(codon)  # it's a stop codon
             return x
         try:
             x = self.forward_table[codon]
@@ -630,10 +686,10 @@ class AmbiguousForwardTable:
                                               self.ambiguous_nucleotide)
         except KeyError:
             self._cache[codon] = KeyError
-            raise KeyError, codon  # stop codon
+            raise KeyError(codon)  # stop codon
         except TranslationError:
             self._cache[codon] = TranslationError
-            raise TranslationError, codon  # does not code
+            raise TranslationError(codon)  # does not code
         assert len(possible) > 0, "unambiguous codons must code"
 
         # Hah!  Only one possible protein, so use it
@@ -657,7 +713,7 @@ class AmbiguousForwardTable:
         # No amino acid encoding for the results
         if len(possible) == 0:
             self._cache[codon] = TranslationError
-            raise TranslationError, codon   # no valid translation
+            raise TranslationError(codon)   # no valid translation
 
         # All of these are valid, so choose one
         # To be unique, sort by smallet ambiguity then alphabetically
@@ -673,7 +729,7 @@ class AmbiguousForwardTable:
         self._cache[codon] = x
         return x
 
-
+#Prepare the ambiguous tables for DNA, RNA and Generic (DNA or RNA)
 ambiguous_dna_by_name = {}
 for key, val in unambiguous_dna_by_name.items():
     ambiguous_dna_by_name[key] = AmbiguousCodonTable(val,
@@ -705,9 +761,8 @@ for key, val in unambiguous_rna_by_id.items():
                                      IUPACData.extended_protein_values)
 
 #The following isn't very elegant, but seems to work nicely.
-_merged_values = dict()
-_merged_values.update(IUPACData.ambiguous_rna_values)
-_merged_values.update(IUPACData.ambiguous_dna_values)
+_merged_values = dict(IUPACData.ambiguous_rna_values.iteritems())
+_merged_values["T"] = "U"
 
 for key, val in generic_by_name.items():
     ambiguous_generic_by_name[key] = AmbiguousCodonTable(val,
@@ -722,6 +777,27 @@ for key, val in generic_by_id.items():
                                      _merged_values,
                                      IUPAC.extended_protein,
                                      IUPACData.extended_protein_values)
-
 del _merged_values
 del key, val
+
+#Basic sanity test,
+for n in ambiguous_generic_by_id.keys() :
+    assert ambiguous_rna_by_id[n].forward_table["GUU"] == "V"
+    assert ambiguous_rna_by_id[n].forward_table["GUN"] == "V"
+    assert ambiguous_rna_by_id[n].forward_table["UUN"] == "X" #F or L
+    #R = A or G, so URR = UAA or UGA / TRA = TAA or TGA = stop codons
+    if "UAA" in unambiguous_rna_by_id[n].stop_codons \
+    and "UGA" in unambiguous_rna_by_id[n].stop_codons :
+        try :
+            print ambiguous_dna_by_id[n].forward_table["TRA"]
+            assert False, "Should be a stop only"
+        except KeyError :
+            pass
+        assert "URA" in ambiguous_generic_by_id[n].stop_codons
+        assert "URA" in ambiguous_rna_by_id[n].stop_codons
+        assert "TRA" in ambiguous_generic_by_id[n].stop_codons
+        assert "TRA" in ambiguous_dna_by_id[n].stop_codons
+del n
+assert ambiguous_generic_by_id[1].stop_codons == ambiguous_generic_by_name["Standard"].stop_codons
+assert ambiguous_generic_by_id[4].stop_codons == ambiguous_generic_by_name["SGC3"].stop_codons
+assert ambiguous_generic_by_id[15].stop_codons == ambiguous_generic_by_name['Blepharisma Macronuclear'].stop_codons
