@@ -5,7 +5,7 @@ programs.
 """
 
 from Bio import Application
-from Bio.Application import _Option, _Argument
+from Bio.Application import _Option
 
 class Primer3Commandline(Application.AbstractCommandline):
     """Commandline object for the Primer3 interface from EMBOSS.
@@ -15,7 +15,7 @@ class Primer3Commandline(Application.AbstractCommandline):
         self.program_name = cmd
 
         self.parameters = \
-          [_Option(["-sequence"], ["input"], None, 1, 
+          [_Option(["-sequence"], ["input"], None, 1,
                    "Sequence to choose primers from"),
            _Option(["-outfile"], ["output", "file"], None, 1,
                    "Output file name"),
@@ -60,7 +60,11 @@ class Primer3Commandline(Application.AbstractCommandline):
            _Option(["-oligodnaconc"], ["input"], None, 0),
            _Option(["-oligoselfany"], ["input"], None, 0),
            _Option(["-oligoselfend"], ["input"], None, 0),
-           _Option(["-oligomaxpolyx"], ["input"], None, 0)]
+           _Option(["-oligomaxpolyx"], ["input"], None, 0),
+           _Option(["-mispriminglibraryfile"], ["input"], None, 0),
+           _Option(["-maxmispriming"], ["input"], None, 0),
+           _Option(["-oligomishyblibraryfile"], ["input"], None, 0),
+           _Option(["-oligomaxmishyb"], ["input"], None, 0),]
 
 class PrimerSearchCommandline(Application.AbstractCommandline):
     """Commandline object for the primersearch program from EMBOSS.
@@ -408,6 +412,54 @@ class ETandemCommandline(Application.AbstractCommandline):
          _Option(["-rformat"], ["output"], None, 0,
                  "Output report format")]
 
+class EInvertedCommandline(Application.AbstractCommandline):
+    """Commandline object for the einverted program from EMBOSS.
+    """
+    def __init__(self, cmd = "einverted"):
+        Application.AbstractCommandline.__init__(self)
+        self.program_name = cmd
+
+        self.parameters = [
+         _Option(["-sequence"], ["input", "file"], None, 1,
+                 "Sequence"),
+         _Option(["-gap"], ["input", "file"], None, 1,
+                 "Gap penalty"),
+         _Option(["-threshold"], ["input"], None, 1,
+                 "Minimum score threshold"),
+         _Option(["-match"], ["input"], None, 1,
+                 "Match score"),
+         _Option(["-mismatch"], ["input"], None, 1,
+                   "Mismatch score"),
+         _Option(["-outfile"], ["output", "file"] , None, 1,
+                 "Output report file name"),
+         _Option(["-maxrepeat"], ["input"], None, 0,
+                 "Maximum separation between the start and end of repeat"),
+         ]
+
+class PalindromeCommandline(Application.AbstractCommandline):
+    """Commandline object for the palindrome program from EMBOSS.
+    """
+    def __init__(self, cmd = "palindrome"):
+        Application.AbstractCommandline.__init__(self)
+        self.program_name = cmd
+
+        self.parameters = [
+         _Option(["-sequence"], ["input", "file"], None, 1,
+                 "Sequence"),
+         _Option(["-minpallen"], ["input"], None, 1,
+                 "Minimum palindrome length"),
+         _Option(["-maxpallen"], ["input"], None, 1,
+                 "Maximum palindrome length"),
+         _Option(["-gaplimit"], ["input"], None, 1,
+                 "Maximum gap between repeats"),
+         _Option(["-nummismatches"], ["input"], None, 1,
+                 "Number of mismatches allowed"),
+         _Option(["-overlap"], ["input"], None, 1,
+                 "Report overlapping matches"),
+         _Option(["-outfile"], ["output", "file"] , None, 1,
+                 "Output report file name"),
+         ]
+
 class TranalignCommandline(Application.AbstractCommandline):
     """Commandline object for the tranalign program from EMBOSS.
     """
@@ -424,3 +476,28 @@ class TranalignCommandline(Application.AbstractCommandline):
                  "Output sequence file."),
          _Option(["-table"], ["input"], None, 0,
                  "Code to use")]
+
+class DiffseqCommandline(Application.AbstractCommandline):
+    """Commandline object for the diffseq program from EMBOSS.
+    """
+    def __init__(self, cmd = "diffseq"):
+        Application.AbstractCommandline.__init__(self)
+        self.program_name = cmd
+
+        self.parameters = [
+         _Option(["-asequence"], ["input", "file"], None, 1,
+                 "First sequence to compare"),
+         _Option(["-bsequence"], ["input", "file"], None, 1,
+                 "Second sequence to compare"),
+         _Option(["-wordsize"], ["input"], None, 1,
+                 "Word size to use for comparisons (10 default)"),
+         _Option(["-outfile"], ["output", "file"], None, 1,
+                "Output report file name"),
+         _Option(["-aoutfeat"], ["output", "file"], None, 1,
+                "File for output of first sequence's features"),
+         _Option(["-boutfeat"], ["output", "file"], None, 1,
+                "File for output of second sequence's features"),
+         _Option(["-rformat"], ["output"], None, 0,
+                 "Output report file format")
+         ]
+
