@@ -9,9 +9,7 @@
 Bio.GFF.easy: some functions to ease the use of Biopython
 """
 
-from __future__ import generators # requires Python 2.2
-
-__version__ = "$Revision: 1.8 $"
+__version__ = "$Revision: 1.11 $"
 # $Source: /home/repository/biopython/biopython/Bio/GFF/easy.py,v $
 
 import copy
@@ -24,11 +22,7 @@ from Bio import GenBank
 from Bio.Data import IUPACData
 from Bio.Seq import Seq
 
-#from Bio.SeqIO.FASTA import FastaReader, FastaWriter
-#The whole of Bio.SeqIO.FASTA has been deprecated, so
-#we'll use the new Bio.SeqIO functions instead:
 from Bio import SeqIO
-
 from Bio import SeqUtils
 
 import GenericTools
@@ -163,7 +157,7 @@ class Location(GenericTools.VerboseList):
         elif self.join:
             return self[index].findseqname(direction)
         else:
-            raise AttributeError, 'no sequence name'
+            raise AttributeError('no sequence name')
 
     def five_prime(self):
         return self.findside(1)
@@ -564,7 +558,7 @@ def record_sequence(record):
     elif isinstance(record, Bio.GenBank.Record.Record):
         return Seq(record.sequence)
     else:
-        raise TypeError, 'not Bio.SeqRecord.SeqRecord or Bio.GenBank.Record.Record'
+        raise TypeError('not Bio.SeqRecord.SeqRecord or Bio.GenBank.Record.Record')
 
 def record_coords(record, start, end, strand=0, upper=0):
     """
