@@ -3,9 +3,6 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
-from __future__ import generators
-
-from Numeric import Float0
 from copy import copy
 
 from PDBExceptions import PDBConstructionException, PDBException
@@ -78,7 +75,8 @@ class Entity:
         "Add a child to the Entity."
         entity_id=entity.get_id()
         if self.has_id(entity_id):
-            raise PDBConstructionException, "%s defined twice" % str(entity_id)
+            raise PDBConstructionException( \
+                "%s defined twice" % str(entity_id))
         entity.set_parent(self)
         self.child_list.append(entity)
         #self.child_list.sort(self._sort)
