@@ -1,13 +1,13 @@
-# Copyright (C) 2002, Thomas Hamelryck (thamelry@vub.ac.be)
+# Copyright (C) 2002, Thomas Hamelryck (thamelry@binf.ku.dk)
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
-
 # Python stuff
 import sys
 from string import split
-from Numeric import array, Float0
+
+import numpy
 
 # My stuff
 from StructureBuilder import StructureBuilder
@@ -50,20 +50,20 @@ class PDBParser:
 
     # Public methods
 
-    def get_structure(self, id, filename):
+    def get_structure(self, id, file):
         """Return the structure.
 
         Arguments:
         o id - string, the id that will be used for the structure
-        o filename - name of the PDB file
+        o file - name of the PDB file OR an open filehandle
         """
         self.header=None
         self.trailer=None
         # Make a StructureBuilder instance (pass id of structure as parameter)
         self.structure_builder.init_structure(id)
-        file=open(filename)
+        if isinstance(file, basestring):
+            file=open(file)
         self._parse(file.readlines())
-        file.close()
         self.structure_builder.set_header(self.header)
         # Return the Structure instance
         return self.structure_builder.get_structure()
@@ -106,17 +106,23 @@ class PDBParser:
         local_line_counter=0
         structure_builder=self.structure_builder
         current_model_id=0
+        # Flag we have an open model
+        model_open=0
         current_chain_id=None
         current_segid=None
         current_residue_id=None
         current_resname=None
-        structure_builder.init_model(current_model_id)
         for i in range(0, len(coords_trailer)):
             line=coords_trailer[i]
             record_type=line[0:6]
             global_line_counter=self.line_counter+local_line_counter+1
             structure_builder.set_line_counter(global_line_counter)
             if(record_type=='ATOM  ' or record_type=='HETATM'):
+                # Initialize the Model - there was no explicit MODEL record
+                if not model_open:
+                    structure_builder.init_model(current_model_id)
+                    current_model_id+=1
+                    model_open=1
                 fullname=line[12:16]
                 # get rid of whitespace in atom names
                 split_list=split(fullname)
@@ -130,6 +136,10 @@ class PDBParser:
                 altloc=line[16:17]
                 resname=line[17:20]
                 chainid=line[21:22]
+                try:
+                    serial_number=int(line[6:11])
+                except:
+                    serial_number=0
                 resseq=int(split(line[22:26])[0])   # sequence identifier   
                 icode=line[26:27]           # insertion code
                 if record_type=='HETATM':       # hetero atom flag
@@ -144,7 +154,7 @@ class PDBParser:
                 x=float(line[30:38]) 
                 y=float(line[38:46]) 
                 z=float(line[46:54])
-                coord=array((x, y, z), Float0)
+                coord=numpy.array((x, y, z), 'f')
                 # occupancy & B factor
                 occupancy=float(line[54:60])
                 bfactor=float(line[60:66])
@@ -170,33 +180,38 @@ class PDBParser:
                         self._handle_PDB_exception(message, global_line_counter) 
                 # init atom
                 try:
-                    structure_builder.init_atom(name, coord, bfactor, occupancy, altloc, fullname)
+                    structure_builder.init_atom(name, coord, bfactor, occupancy, altloc, fullname, serial_number)
                 except PDBConstructionException, message:
                     self._handle_PDB_exception(message, global_line_counter)
             elif(record_type=='ANISOU'):
                 anisou=map(float, (line[28:35], line[35:42], line[43:49], line[49:56], line[56:63], line[63:70]))
                 # U's are scaled by 10^4 
-                anisou_array=(array(anisou, Float0)/10000.0).astype(Float0)
+                anisou_array=(numpy.array(anisou, 'f')/10000.0).astype('f')
                 structure_builder.set_anisou(anisou_array)
-            elif(record_type=='ENDMDL'):
-                current_model_id=current_model_id+1
+            elif(record_type=='MODEL '):
                 structure_builder.init_model(current_model_id)
+                current_model_id+=1
+                model_open=1
                 current_chain_id=None
                 current_residue_id=None
             elif(record_type=='END   ' or record_type=='CONECT'):
                 # End of atomic data, return the trailer
                 self.line_counter=self.line_counter+local_line_counter
                 return coords_trailer[local_line_counter:]
+            elif(record_type=='ENDMDL'):
+                model_open=0
+                current_chain_id=None
+                current_residue_id=None
             elif(record_type=='SIGUIJ'):
                 # standard deviation of anisotropic B factor
                 siguij=map(float, (line[28:35], line[35:42], line[42:49], line[49:56], line[56:63], line[63:70]))
                 # U sigma's are scaled by 10^4
-                siguij_array=(array(siguij, Float0)/10000.0).astype(Float0)   
+                siguij_array=(numpy.array(siguij, 'f')/10000.0).astype('f')   
                 structure_builder.set_siguij(siguij_array)
             elif(record_type=='SIGATM'):
                 # standard deviation of atomic positions
                 sigatm=map(float, (line[30:38], line[38:45], line[46:54], line[54:60], line[60:66]))
-                sigatm_array=array(sigatm, Float0)
+                sigatm_array=numpy.array(sigatm, 'f')
                 structure_builder.set_sigatm(sigatm_array)
             local_line_counter=local_line_counter+1
         # EOF (does not end in END or CONECT)
@@ -216,7 +231,7 @@ class PDBParser:
             print "Exception ignored.\nSome atoms or residues will be missing in the data structure."
         else:
             # exceptions are fatal - raise again with new message (including line nr)
-            raise PDBConstructionException, message
+            raise PDBConstructionException(message)
 
 
 if __name__=="__main__":
@@ -227,16 +242,17 @@ if __name__=="__main__":
 
     s=p.get_structure("scr", sys.argv[1])
 
-    for m in s.get_iterator():
+    for m in s:
         p=m.get_parent()
         assert(p is s)
-        for c in m.get_iterator():
+        for c in m:
             p=c.get_parent()
             assert(p is m)
-            for r in c.get_iterator():
+            for r in c:
+                print r
                 p=r.get_parent()
                 assert(p is c)
-                for a in r.get_iterator():
+                for a in r:
                     p=a.get_parent()
                     if not p is r:
                         print p, r
