@@ -122,7 +122,7 @@ def parse_domain(str) :
     """
 
     m = _domain_re.match(str)
-    if (not m) : raise ValueError, "Domain: "+ str
+    if (not m) : raise ValueError("Domain: "+ str)
 
     dom = Domain()
     dom.sid = m.group(1)
@@ -167,7 +167,7 @@ class Scop:
 
         if dir_path is None and db_handle is None:
             if cla_handle == None or des_handle==None or hie_handle==None:
-                raise RuntimeError,"Need CLA, DES and HIE files to build SCOP" 
+                raise RuntimeError("Need CLA, DES and HIE files to build SCOP")
 
         sunidDict = {}
 
@@ -182,9 +182,9 @@ class Scop:
                 # open SCOP parseable files 
                 if dir_path:
                     if not version :
-                        raise RuntimeError, "Need SCOP version to find parsable files in directory"
+                        raise RuntimeError("Need SCOP version to find parsable files in directory")
                     if cla_handle or des_handle or hie_handle:
-                        raise RuntimeError,"Cannot specify SCOP directory and specific files"
+                        raise RuntimeError("Cannot specify SCOP directory and specific files")
                 
                     cla_handle = _open_scop_file( dir_path, version, 'cla')
                     des_handle = _open_scop_file( dir_path, version, 'des')
@@ -199,56 +199,50 @@ class Scop:
                 root.description = 'SCOP Root'
 
                 # Build the rest of the nodes using the DES file
-                i = Des.Iterator(des_handle, Des.Parser())
-                while 1 :
-                    rec = i.next() 
-                    if rec is None : break
-                    if rec.nodetype =='px' :
+                records = Des.parse(des_handle)
+                for record in records:
+                    if record.nodetype =='px' :
                         n = Domain()
-                        n.sid = rec.name
+                        n.sid = record.name
                         domains.append(n)
                     else : 
                         n = Node()
-                    n.sunid = rec.sunid
-                    n.type = rec.nodetype
-                    n.sccs = rec.sccs
-                    n.description = rec.description
+                    n.sunid = record.sunid
+                    n.type = record.nodetype
+                    n.sccs = record.sccs
+                    n.description = record.description
                     
                     sunidDict[n.sunid] = n
  
                 # Glue all of the Nodes together using the HIE file
-                i = Hie.Iterator(hie_handle, Hie.Parser())
-                while 1 :
-                    rec = i.next()
-                    if rec is None : break
-                    if not sunidDict.has_key(rec.sunid) :
-                        print rec.sunid
+                records = Hie.parse(hie_handle)
+                for record in records:
+                    if record.sunid not in sunidDict :
+                        print record.sunid
                         
-                    n = sunidDict[rec.sunid]
+                    n = sunidDict[record.sunid]
     
-                    if rec.parent != '' : # Not root node
+                    if record.parent != '' : # Not root node
     
-                        if not sunidDict.has_key(rec.parent):
-                            raise ValueError, "Incomplete data?"
+                        if record.parent not in sunidDict:
+                            raise ValueError("Incomplete data?")
                                        
-                        n.parent = sunidDict[rec.parent]
+                        n.parent = sunidDict[record.parent]
                 
-                    for c in rec.children:
-                        if not sunidDict.has_key(c) :
-                            raise ValueError, "Incomplete data?"
+                    for c in record.children:
+                        if c not in sunidDict :
+                            raise ValueError("Incomplete data?")
                         n.children.append(sunidDict[c])
 
                         
                 # Fill in the gaps with information from the CLA file
                 sidDict = {}
-                i = Cla.Iterator(cla_handle, Cla.Parser())
-                while 1 :
-                    rec = i.next()
-                    if rec is None : break
-                    n = sunidDict[rec.sunid]
-                    assert n.sccs == rec.sccs
-                    assert n.sid == rec.sid
-                    n.residues = rec.residues
+                records = Cla.parse(cla_handle)
+                for record in records:
+                    n = sunidDict[record.sunid]
+                    assert n.sccs == record.sccs
+                    assert n.sid == record.sid
+                    n.residues = record.residues
                     sidDict[n.sid] = n
 
                 # Clean up
@@ -270,11 +264,11 @@ class Scop:
 
     def getDomainBySid(self, sid) :
         """Return a domain from its sid"""
-        if self._sidDict.has_key(sid):
+        if sid in self._sidDict:
             return self._sidDict[sid]
         if self.db_handle:
             self.getDomainFromSQL(sid=sid)
-            if self._sidDict.has_key(sid):
+            if sid in self._sidDict:
                 return self._sidDict[sid]
         else:
             return None
@@ -282,11 +276,11 @@ class Scop:
 
     def getNodeBySunid(self, sunid) :
         """Return a node from its sunid"""
-        if self._sunidDict.has_key(sunid):
+        if sunid in self._sunidDict:
             return self._sunidDict[sunid]
         if self.db_handle:
             self.getDomainFromSQL(sunid=sunid)
-            if self._sunidDict.has_key(sunid):
+            if sunid in self._sunidDict:
                 return self._sunidDict[sunid]
         else:
             return None
@@ -421,7 +415,7 @@ class Scop:
             cla,des WHERE cla."+node.type+"=%s AND cla."+type+"=des.sunid", (node.sunid))
             data = cur.fetchall()
             for d in data:
-                if not self._sunidDict.has_key(int(d[0])):
+                if int(d[0]) not in self._sunidDict:
                     n = Node(scop=self)
                     [n.sunid,n.type,n.sccs,n.description] = d
                     n.sunid=int(n.sunid)
@@ -446,7 +440,7 @@ class Scop:
 
             data = cur.fetchall()
             for d in data:
-                if not self._sunidDict.has_key(int(d[0])):
+                if int(d[0]) not in self._sunidDict:
                     n = Domain(scop=self)
                     #[n.sunid, n.sid, n.pdbid, n.residues, n.sccs, n.type,
                     #n.description,n.parent] = data
@@ -590,7 +584,7 @@ class Node :
         """ Return a list of all decendent nodes of the given type. Node type can a
         two letter code or longer description. e.g. 'fa' or 'family'
         """
-        if _nodetype_to_code.has_key(node_type):
+        if node_type in _nodetype_to_code:
             node_type = _nodetype_to_code[node_type]
             
         nodes = [self]
@@ -610,7 +604,7 @@ class Node :
     def getAscendent( self, node_type) :
         """ Return the ancenstor node of the given type, or None.Node type can a
         two letter code or longer description. e.g. 'fa' or 'family'"""
-        if _nodetype_to_code.has_key(node_type):
+        if node_type in _nodetype_to_code:
             node_type = _nodetype_to_code[node_type]
 
         if self.scop:
@@ -707,13 +701,13 @@ class Astral:
         Note that the ind_file argument is deprecated.
         """
         if ind_file :
-            raise RuntimeError, "The ind_file (index file) argument is deprecated"
+            raise RuntimeError("The ind_file (index file) argument is deprecated")
 
         if astral_file==dir_path==db_handle==None:
-            raise RunTimeError,"Need either file handle, or (dir_path + version)\
-            or database handle to construct Astral"
+            raise RuntimeError("Need either file handle, or (dir_path + "\
+                       + "version) or database handle to construct Astral")
         if not scop:
-            raise RuntimeError, "Must provide a Scop instance to construct"
+            raise RuntimeError("Must provide a Scop instance to construct")
 
         self.scop = scop
         self.db_handle = db_handle 
@@ -721,7 +715,7 @@ class Astral:
         
         if not astral_file and not db_handle:
             if dir_path == None or version == None:
-                raise RuntimeError, "must provide dir_path and version"
+                raise RuntimeError("must provide dir_path and version")
 
             self.version = version
             self.path = os.path.join( dir_path, "scopseq-%s" % version)
@@ -741,13 +735,13 @@ class Astral:
         
     def domainsClusteredByEv(self,id):
         """get domains clustered by evalue"""
-        if not self.EvDatasets.has_key(id):
+        if id not in self.EvDatasets:
             if self.db_handle:
                 self.EvDatasets[id] = self.getAstralDomainsFromSQL(astralEv_to_sql[id])
                 
             else:
                 if not self.path:
-                    raise RuntimeError, "No scopseq directory specified"
+                    raise RuntimeError("No scopseq directory specified")
                 
                 file_prefix = "astral-scopdom-seqres-sel-gs"
                 filename = "%s-e100m-%s-%s.id" % (file_prefix, astralEv_to_file[id] ,
@@ -759,13 +753,13 @@ class Astral:
 
     def domainsClusteredById(self,id):
         """get domains clustered by percent id"""
-        if not self.IdDatasets.has_key(id):
+        if id not in self.IdDatasets:
             if self.db_handle:
                 self.IdDatasets[id] = self.getAstralDomainsFromSQL("id"+str(id))
                 
             else:
                 if not self.path:
-                    raise RuntimeError, "No scopseq directory specified"
+                    raise RuntimeError("No scopseq directory specified")
                 
                 file_prefix = "astral-scopdom-seqres-sel-gs"
                 filename = "%s-bib-%s-%s.id" % (file_prefix, id, self.version)
@@ -777,7 +771,7 @@ class Astral:
     def getAstralDomainsFromFile(self,filename=None,file_handle=None):
         """Get the scop domains from a file containing a list of sids"""
         if file_handle == filename == none:
-            raise RuntimeError, "You must provide a filename or handle"
+            raise RuntimeError("You must provide a filename or handle")
         if not file_handle:
             file_handle = open(filename)
         doms = []
@@ -822,7 +816,7 @@ class Astral:
 
     def hashedDomainsById(self,id):
         """Get domains clustered by sequence identity in a dict"""
-        if not self.IdDatahash.has_key(id):
+        if id not in self.IdDatahash:
             self.IdDatahash[id] = {}
             for d in self.domainsClusteredById(id):
                 self.IdDatahash[id][d] = 1
@@ -830,7 +824,7 @@ class Astral:
 
     def hashedDomainsByEv(self,id):
         """Get domains clustered by evalue in a dict"""
-        if not self.EvDatahash.has_key(id):
+        if id not in self.EvDatahash:
             self.EvDatahash[id] = {}
             for d in self.domainsClusteredByEv(id):
                 self.EvDatahash[id][d] = 1
@@ -839,11 +833,11 @@ class Astral:
 
     def isDomainInId(self,dom,id):
         """Returns true if the domain is in the astral clusters for percent ID"""
-        return self.hashedDomainsById(id).has_key(dom)
+        return dom in self.hashedDomainsById(id)
 
     def isDomainInEv(self,dom,id):
         """Returns true if the domain is in the ASTRAL clusters for evalues"""
-        return self.hashedDomainsByEv(id).has_key(dom)
+        return dom in self.hashedDomainsByEv(id)
             
 
     def writeToSQL(self, db_handle):
