@@ -1,9 +1,9 @@
-# Copyright 2006, 2007 by Peter Cock.  All rights reserved.
+# Copyright 2006-2008 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 #
-# This module is for reading and writting FASTA format files as SeqRecord
+# This module is for reading and writing FASTA format files as SeqRecord
 # objects.  The code is partly inspired  by earlier Biopython modules,
 # Bio.Fasta.* and the now deprecated Bio.SeqIO.FASTA
 
@@ -40,7 +40,7 @@ def FastaIterator(handle, alphabet = single_letter_alphabet, title2ids = None) :
             break
 
     while True :
-        if line[0]<>">" :
+        if line[0]!=">" :
             raise ValueError("Records in Fasta files should start with '>' character")
         if title2ids :
             id, name, descr = title2ids(line[1:].rstrip())
@@ -48,16 +48,18 @@ def FastaIterator(handle, alphabet = single_letter_alphabet, title2ids = None) :
             descr = line[1:].rstrip()
             id   = descr.split()[0]
             name = id
-            
+
         lines = []
         line = handle.readline()
         while True:
             if not line : break
             if line[0] == ">": break
             #Remove trailing whitespace, and any internal spaces
-            lines.append(line.rstrip().replace(" ",""))
+            #(and any embedded \r which are possible in mangled files
+            #when not opened in universal read lines mode)
+            lines.append(line.rstrip().replace(" ","").replace("\r",""))
             line = handle.readline()
-            
+
         #Return the record and then continue...
         yield SeqRecord(Seq("".join(lines), alphabet),
                          id = id, name = name, description = descr)
@@ -66,7 +68,7 @@ def FastaIterator(handle, alphabet = single_letter_alphabet, title2ids = None) :
     assert False, "Should not reach this line"
 
 class FastaWriter(SequentialSequenceWriter):
-    """Class to write Fasta format files"""
+    """Class to write Fasta format files."""
     def __init__(self, handle, wrap=60, record2title=None):
         """Create a Fasta writer.
 
@@ -107,13 +109,13 @@ class FastaWriter(SequentialSequenceWriter):
         self.record2title = record2title
 
     def write_record(self, record):
-        """Write a single Fasta record to the file"""
+        """Write a single Fasta record to the file."""
         assert self._header_written
         assert not self._footer_written
         self._record_written = True
-        
+
         if self.record2title :
-            title=self.clean(record2title(record))
+            title=self.clean(self.record2title(record))
         else :
             id = self.clean(record.id)
             description = self.clean(record.description)
@@ -141,15 +143,15 @@ class FastaWriter(SequentialSequenceWriter):
 
 if __name__ == "__main__" :
     print "Running quick self test"
-    
+
     import os
     from Bio.Alphabet import generic_protein, generic_nucleotide
-    
+
     #Download the files from here:
     #ftp://ftp.ncbi.nlm.nih.gov/genomes/Bacteria/Nanoarchaeum_equitans
     fna_filename = "NC_005213.fna"
     faa_filename = "NC_005213.faa"
-        
+
     def genbank_name_function(text) :
         text, descr = text.split(None,1)
         id = text.split("|")[3]
