@@ -142,8 +142,12 @@ class Iterator:
         If set to None, then the raw contents of the file will be returned.
 
         """
+        import warnings
+        warnings.warn("Bio.SwissProt.SProt.Iterator is deprecated. Please use the function Bio.SwissProt.parse instead if you want to get a SwissProt.SProt.Record, or Bio.SeqIO.parse if you want to get a SeqRecord. If these solutions do not work for you, please get in contact with the Biopython developers (biopython-dev@biopython.org).",
+              DeprecationWarning)
+
         if type(handle) is not FileType and type(handle) is not InstanceType:
-            raise ValueError, "I expected a file handle or file-like object"
+            raise ValueError("I expected a file handle or file-like object")
         self._uhandle = File.UndoHandle(handle)
         self._parser = parser
 
@@ -236,21 +240,21 @@ class ExPASyDictionary:
         self.limiter = RequestLimiter(delay)
 
     def __len__(self):
-        raise NotImplementedError, "SwissProt contains lots of entries"
+        raise NotImplementedError("SwissProt contains lots of entries")
     def clear(self):
-        raise NotImplementedError, "This is a read-only dictionary"
+        raise NotImplementedError("This is a read-only dictionary")
     def __setitem__(self, key, item):
-        raise NotImplementedError, "This is a read-only dictionary"
+        raise NotImplementedError("This is a read-only dictionary")
     def update(self):
-        raise NotImplementedError, "This is a read-only dictionary"
+        raise NotImplementedError("This is a read-only dictionary")
     def copy(self):
-        raise NotImplementedError, "You don't need to do this..."
+        raise NotImplementedError("You don't need to do this...")
     def keys(self):
-        raise NotImplementedError, "You don't really want to do this..."
+        raise NotImplementedError("You don't really want to do this...")
     def items(self):
-        raise NotImplementedError, "You don't really want to do this..."
+        raise NotImplementedError("You don't really want to do this...")
     def values(self):
-        raise NotImplementedError, "You don't really want to do this..."
+        raise NotImplementedError("You don't really want to do this...")
     
     def has_key(self, id):
         """has_key(self, id) -> bool"""
@@ -265,7 +269,6 @@ class ExPASyDictionary:
             return self[id]
         except KeyError:
             return failobj
-        raise "How did I get here?"
 
     def __getitem__(self, id):
         """__getitem__(self, id) -> object
@@ -274,7 +277,7 @@ class ExPASyDictionary:
         for the entry.  Raises a KeyError if there's an error.
         
         """
-        from Bio.WWW import ExPASy
+        from Bio import ExPASy
         # First, check to see if enough time has passed since my
         # last query.
         self.limiter.wait()
@@ -282,7 +285,7 @@ class ExPASyDictionary:
         try:
             handle = ExPASy.get_sprot_raw(id)
         except IOError:
-            raise KeyError, id
+            raise KeyError(id)
         
         if self.parser is not None:
             return self.parser.parse(handle)
@@ -337,9 +340,8 @@ class _Scanner:
             uhandle = handle
         else:
             uhandle = File.UndoHandle(handle)
-        
         self._scan_record(uhandle, consumer)
-
+        
     def _skip_starstar(self, uhandle) :
         """Ignores any lines starting **"""
         #See Bug 2353, some files from the EBI have extra lines
@@ -430,7 +432,7 @@ class _Scanner:
         self._scan_line('OH', uhandle, consumer.organism_host, any_number=1)
 
     def _scan_reference(self, uhandle, consumer):
-        while 1:
+        while True:
             if safe_peekline(uhandle)[:2] != 'RN':
                 break
             self._scan_rn(uhandle, consumer)
@@ -549,9 +551,11 @@ class _RecordConsumer(AbstractConsumer):
         
     def start_record(self):
         self.data = Record()
+        self._sequence_lines = []
         
     def end_record(self):
         self._clean_record(self.data)
+        self.data.sequence = "".join(self._sequence_lines)
 
     def identification(self, line):
         cols = line.split()
@@ -582,14 +586,14 @@ class _RecordConsumer(AbstractConsumer):
         # pjc:2006-11-02 added 'Reviewed' and 'Unreviewed'
         if self.data.data_class not in ['STANDARD', 'PRELIMINARY', 'IPI',
                                         'Reviewed', 'Unreviewed']: 
-            raise ValueError, "Unrecognized data class %s in line\n%s" % \
-                  (self.data.data_class, line)
+            raise ValueError("Unrecognized data class %s in line\n%s" % \
+                  (self.data.data_class, line))
         # molecule_type should be 'PRT' for PRoTein
         # Note that has been removed in recent releases (set to None)
         if self.data.molecule_type is not None \
         and self.data.molecule_type != 'PRT':
-            raise ValueError, "Unrecognized molecule type %s in line\n%s" % \
-                  (self.data.molecule_type, line)
+            raise ValueError("Unrecognized molecule type %s in line\n%s" % \
+                  (self.data.molecule_type, line))
     
     def accession(self, line):
         cols = line[5:].rstrip(_CHOMP).strip().split(';')
@@ -599,7 +603,7 @@ class _RecordConsumer(AbstractConsumer):
                 self.data.accessions.append(ac.strip())
     
     def date(self, line):
-        uprline = string.upper(line)
+        uprline = line.upper()
         cols = line.rstrip().split()
                 
         if uprline.find('CREATED') >= 0 \
@@ -687,7 +691,7 @@ class _RecordConsumer(AbstractConsumer):
             else:
                 assert False, "Shouldn't reach this line!"
         else:
-            raise ValueError, "I don't understand the date line %s" % line
+            raise ValueError("I don't understand the date line %s" % line)
     
     def description(self, line):
         self.data.description += line[5:]
@@ -940,8 +944,8 @@ class _RecordConsumer(AbstractConsumer):
         self.data.seqinfo = int(cols[2]), int(cols[4]), cols[6]
     
     def sequence_data(self, line):
-        seq = line.replace(" ", "").rstrip()
-        self.data.sequence = self.data.sequence + seq
+        #It should be faster to make a list of strings, and join them at the end.
+        self._sequence_lines.append(line.replace(" ", "").rstrip())
     
     def terminator(self, line):
         pass
@@ -991,12 +995,14 @@ class _SequenceConsumer(AbstractConsumer):
         self.data.description = ""
         self.data.name = ""
         self._current_ref = None
+        self._sequence_lines = []
         
     def end_record(self):
         if self._current_ref is not None:
             self.data.annotations['references'].append(self._current_ref)
             self._current_ref = None
         self.data.description = self.data.description.rstrip()
+        self.data.seq = Seq.Seq("".join(self._sequence_lines), self.alphabet)
 
     def identification(self, line):
         cols = line.split()
@@ -1023,9 +1029,8 @@ class _SequenceConsumer(AbstractConsumer):
                                 line[5:].strip() + "\n"
         
     def sequence_data(self, line):
-        seq = Seq.Seq(line.replace(" ", "").rstrip(),
-                      self.alphabet)
-        self.data.seq = self.data.seq + seq
+        #It should be faster to make a list of strings, and join them at the end.
+        self._sequence_lines.append(line.replace(" ", "").rstrip())
 
     def gene_name(self, line):
         #We already store the identification/accession as the records name/id
@@ -1068,7 +1073,7 @@ class _SequenceConsumer(AbstractConsumer):
                     
     def date(self, line):
         date_str = line.split()[0]
-        uprline = string.upper(line)
+        uprline = line.upper()
         if uprline.find('CREATED') >= 0 :
             #Try and agree with SeqRecord convention from the GenBank parser,
             #which stores the submitted date as 'date'
@@ -1254,57 +1259,29 @@ def index_file(filename, indexname, rec2key=None):
     the entry name will be used.
 
     """
+    from Bio.SwissProt import parse
     if not os.path.exists(filename):
-        raise ValueError, "%s does not exist" % filename
+        raise ValueError("%s does not exist" % filename)
     
     index = Index.Index(indexname, truncate=1)
     index[Dictionary._Dictionary__filename_key] = filename
     
-    iter = Iterator(open(filename), parser=RecordParser())
-    while 1:
-        start = iter._uhandle.tell()
-        rec = iter.next()
-        length = iter._uhandle.tell() - start
+    handle = open(filename)
+    records = parse(handle)
+    end = 0L
+    for record in records:
+        start = end
+        end = long(handle.tell())
+        length = end - start
         
-        if rec is None:
-            break
         if rec2key is not None:
-            key = rec2key(rec)
+            key = rec2key(record)
         else:
-            key = rec.entry_name
-            
+            key = record.entry_name
+
         if not key:
-            raise KeyError, "empty sequence key was produced"
-        elif index.has_key(key):
-            raise KeyError, "duplicate key %s found" % key
+            raise KeyError("empty sequence key was produced")
+        elif key in index:
+            raise KeyError("duplicate key %s found" % key)
 
         index[key] = start, length
-        
-if __name__ == "__main__" :
-    print "Quick self test..."
-
-    example_filename = "../../Tests/SwissProt/sp008"
-
-    import os
-    if not os.path.isfile(example_filename) :
-        print "Missing test file %s" % example_filename
-    else :
-        #Try parsing it!
-        
-        handle = open(example_filename)
-        for record in Iterator(handle, RecordParser()) :
-            print record.entry_name
-            print ",".join(record.accessions)
-            print record.keywords
-            print repr(record.organism)
-            print record.sequence[:20] + "..."
-        handle.close()
-
-        handle = open(example_filename)
-        for record in Iterator(handle, SequenceParser()) :
-            print record.name
-            print record.id
-            print record.annotations['keywords']
-            print repr(record.annotations['organism'])
-            print record.seq.tostring()[:20] + "..."
-        handle.close()
