@@ -27,11 +27,33 @@ def read(handle):
         record = parser.parse(handle)
     except ValueError, error:
         if error.message.startswith("Line does not start with 'ID':"): 
-            raise ValueError, "No SwissProt record found"
+            raise ValueError("No SwissProt record found")
         else:
             raise error
     # We should have reached the end of the record by now
     remainder = handle.read()
     if remainder:
-        raise ValueError, "More than one SwissProt record found"
+        raise ValueError("More than one SwissProt record found")
     return record
+
+
+if __name__ == "__main__" :
+    print "Quick self test..."
+
+    example_filename = "../../Tests/SwissProt/sp008"
+
+    import os
+    if not os.path.isfile(example_filename):
+        print "Missing test file %s" % example_filename
+    else :
+        #Try parsing it!
+        
+        handle = open(example_filename)
+        records = parse(handle)
+        for record in records:
+            print record.entry_name
+            print ",".join(record.accessions)
+            print record.keywords
+            print repr(record.organism)
+            print record.sequence[:20] + "..."
+        handle.close()
