@@ -1,5 +1,6 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
-# Revisions 2007-2008 by Peter Cock.
+# Revisions 2007-2008 copyright by Peter Cock.  All rights reserved.
+# Revisions 2008 copyright by Cymon J. Cox.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -12,12 +13,18 @@ This allows retrival of items stored in a BioSQL database using
 a biopython-like Seq interface.
 """
 
+from Bio import Alphabet
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 from Bio import SeqFeature
 
 class DBSeq(Seq):  # This implements the biopython Seq interface
     def __init__(self, primary_id, adaptor, alphabet, start, length):
+        """Create a new DBSeq object referring to a BioSQL entry.
+
+        You wouldn't normally create a DBSeq object yourself, this is done
+        for you when retreiving a DBSeqRecord object from the database.
+        """
         self.primary_id = primary_id
         self.adaptor = adaptor
         self.alphabet = alphabet
@@ -44,7 +51,7 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
                                                      self.start + i,
                                                      self.start + i + 1)
         if not isinstance(index, slice) :
-            raise ValueError, "Unexpected index type"
+            raise ValueError("Unexpected index type")
 
         #Return the (sub)sequence as another DBSeq or Seq object
         #(see the Seq obect's __getitem__ method)
@@ -62,7 +69,7 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
             i = self._length
 
         if index.stop is None :
-            j = -1
+            j = self._length
         else :
             j = index.stop
         if j < 0 :
@@ -88,11 +95,34 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
             return Seq(full[::index.step], self.alphabet)
         
     def tostring(self):
+        """Returns the full sequence as a python string.
+
+        Although not formally deprecated, you are now encouraged to use
+        str(my_seq) instead of my_seq.tostring()."""
+        return self.adaptor.get_subseq_as_string(self.primary_id,
+                                                 self.start,
+                                                 self.start + self._length)
+    def __str__(self):
+        """Returns the full sequence as a python string."""
         return self.adaptor.get_subseq_as_string(self.primary_id,
                                                  self.start,
                                                  self.start + self._length)
 
-    data = property(tostring)
+    data = property(tostring, doc="Sequence as string (DEPRECATED)")
+
+    def toseq(self):
+        """Returns the full sequence as a Seq object."""
+        #Note - the method name copies that of the MutableSeq object
+        return Seq(str(self), self.alphabet)
+
+    def __add__(self, other) :
+        #Let the Seq object deal with the alphabet issues etc
+        return self.toseq() + other
+
+    def __radd__(self, other) :
+        #Let the Seq object deal with the alphabet issues etc
+        return other + self.toseq()
+
 
 def _retrieve_seq(adaptor, primary_id):
     seqs = adaptor.execute_and_fetchall(
@@ -101,13 +131,18 @@ def _retrieve_seq(adaptor, primary_id):
     if seqs:
         moltype, length = seqs[0]
         moltype = moltype.lower() #might be upper case in database
-        from Bio.Alphabet import IUPAC
+        #We have no way of knowing if these sequences will use IUPAC
+        #alphabets, and we certainly can't assume they are unambiguous!
         if moltype == "dna":
-            alphabet = IUPAC.unambiguous_dna
+            alphabet = Alphabet.generic_dna
         elif moltype == "rna":
-            alphabet = IUPAC.unambiguous_rna
+            alphabet = Alphabet.generic_rna
         elif moltype == "protein":
-            alphabet = IUPAC.protein
+            alphabet = Alphabet.generic_protein
+        elif moltype == "unknown":
+            #This is used in BioSQL/Loader.py and would happen
+            #for any generic or nucleotide alphabets.
+            alphabet = Alphabet.single_letter_alphabet
         else:
             raise AssertionError("Unknown moltype: %s" % moltype)
         seq = DBSeq(primary_id, adaptor, alphabet, 0, int(length))
@@ -143,7 +178,8 @@ def _retrieve_features(adaptor, primary_id):
         qvs = adaptor.execute_and_fetchall(
             "SELECT name, value" \
             " FROM seqfeature_qualifier_value  join term using (term_id)" \
-            " WHERE seqfeature_id = %s", (seqfeature_id,))
+            " WHERE seqfeature_id = %s" \
+            " ORDER BY rank", (seqfeature_id,))
         qualifiers = {}
         for qv_name, qv_value in qvs:
             qualifiers.setdefault(qv_name, []).append(qv_value)
@@ -151,7 +187,8 @@ def _retrieve_features(adaptor, primary_id):
         qvs = adaptor.execute_and_fetchall(
             "SELECT dbxref.dbname, dbxref.accession" \
             " FROM dbxref join seqfeature_dbxref using (dbxref_id)" \
-            " WHERE seqfeature_dbxref.seqfeature_id = %s", (seqfeature_id,))
+            " WHERE seqfeature_dbxref.seqfeature_id = %s" \
+            " ORDER BY rank", (seqfeature_id,))
         for qv_name, qv_value in qvs:
             value = "%s:%s" % (qv_name, qv_value)
             qualifiers.setdefault("db_xref", []).append(value)
@@ -163,9 +200,16 @@ def _retrieve_features(adaptor, primary_id):
             " ORDER BY rank", (seqfeature_id,))
         locations = []
         # convert to Python standard form
+        # Convert strand = 0 to strand = None
+        # re: comment in Loader.py:
+        # Biopython uses None when we don't know strand information but
+        # BioSQL requires something (non null) and sets this as zero
+        # So we'll use the strand or 0 if Biopython spits out None
         for location_id, start, end, strand in results:
             if start:
                 start -= 1
+            if strand == 0:
+                strand = None
             locations.append( (location_id, start, end, strand) )
         # Get possible remote reference information
         remote_results = adaptor.execute_and_fetchall(
@@ -178,60 +222,75 @@ def _retrieve_features(adaptor, primary_id):
                 v = "%s.%s" % (accession, version)
             else:
                 v = accession
+            # subfeature remote location db_ref are stored as a empty string when
+            # not present
+            if dbname == "":
+                dbname = None
             lookup[location_id] = (dbname, v)
         
         feature = SeqFeature.SeqFeature(type = seqfeature_type)
+        feature._seqfeature_id = seqfeature_id #Store the key as a private property
         feature.qualifiers = qualifiers
         if len(locations) == 0:
             pass
         elif len(locations) == 1:
             location_id, start, end, strand = locations[0]
+            #See Bug 2677, we currently don't record the location_operator
+            #For consistency with older versions Biopython, default to "".
+            feature.location_operator = \
+                _retrieve_location_qualifier_value(adaptor, location_id)
             dbname, version = lookup.get(location_id, (None, None))
-
             feature.location = SeqFeature.FeatureLocation(start, end)
             feature.strand = strand
             feature.ref_db = dbname
             feature.ref = version
         else:
-            min_start = locations[0][1]
-            max_end = locations[0][2]
-            sub_feature_list = []       # (start, sub feature) for sorting
+            assert feature.sub_features == []
             for location in locations:
                 location_id, start, end, strand = location
                 dbname, version = lookup.get(location_id, (None, None))
-                min_start = min(min_start, start)
-                max_end = max(max_end, end)
-
                 subfeature = SeqFeature.SeqFeature()
-                subfeature.type = seqfeature_type 
-                subfeature.location_operator = "join"
+                subfeature.type = seqfeature_type
+                subfeature.location_operator = \
+                    _retrieve_location_qualifier_value(adaptor, location_id)
+                #TODO - See Bug 2677 - we don't yet record location_operator,
+                #so for consistency with older versions of Biopython default
+                #to assuming its a join.
+                if not subfeature.location_operator :
+                    subfeature.location_operator="join"
                 subfeature.location = SeqFeature.FeatureLocation(start, end)
                 subfeature.strand = strand
                 subfeature.ref_db = dbname
                 subfeature.ref = version
-                sub_feature_list.append((start, subfeature))
-            sub_feature_list.sort()
-            feature.sub_features = [sub_feature[1]
-                                    for sub_feature in sub_feature_list]
-            feature.location = SeqFeature.FeatureLocation(min_start, max_end)
+                feature.sub_features.append(subfeature)
+            # Assuming that the feature loc.op is the same as the sub_feature
+            # loc.op:
+            feature.location_operator = \
+                feature.sub_features[0].location_operator
+            # Locations are in order, but because of remote locations for
+            # sub-features they are not necessarily in numerical order:
+            start = locations[0][1]
+            end = locations[-1][2]
+            feature.location = SeqFeature.FeatureLocation(start, end)
             feature.strand = feature.sub_features[0].strand
 
-        seq_feature_list.append(
-            (seqfeature_rank, feature.location.start.position, feature) )
+        seq_feature_list.append(feature)
 
-    # Primary sort is on the feature's rank
-    #  .. then on the start position
-    #  .. then arbitrary on the feature's id (SeqFeature has no __cmp__)
-    seq_feature_list.sort()
+    return seq_feature_list
 
-    # Get just the SeqFeature
-    return [x[2] for x in seq_feature_list]
+def _retrieve_location_qualifier_value(adaptor, location_id):
+    value = adaptor.execute_and_fetch_col0(
+        "SELECT value FROM location_qualifier_value" \
+        " WHERE location_id = %s", (location_id,))
+    try:
+        return value[0] 
+    except IndexError:
+        return ""
 
 def _retrieve_annotations(adaptor, primary_id, taxon_id):
     annotations = {}
     annotations.update(_retrieve_qualifier_value(adaptor, primary_id))
     annotations.update(_retrieve_reference(adaptor, primary_id))
-    #annotations.update(_retrieve_dbxref(adaptor, primary_id))
     annotations.update(_retrieve_taxon(adaptor, primary_id, taxon_id))
     return annotations
 
@@ -266,8 +325,8 @@ def _retrieve_reference(adaptor, primary_id):
         reference = SeqFeature.Reference()
         if start: start -= 1
         reference.location = [SeqFeature.FeatureLocation(start, end)]
-        reference.authors = authors
         #Don't replace the default "" with None.
+        if authors : reference.authors = authors
         if title : reference.title = title
         reference.journal = location
         if dbname == 'PUBMED':
@@ -277,26 +336,6 @@ def _retrieve_reference(adaptor, primary_id):
         references.append(reference)
     return {'references': references}
 
-#Why did this exist? The dbxrefs list is already accessable
-#via the record's .dbxrefs property as expected in a SeqRecord
-#
-#def _retrieve_dbxref(adaptor, primary_id):
-#    # XXX dbxref_qualifier_value
-#
-#    refs = adaptor.execute_and_fetchall(
-#        "SELECT dbname, accession, version" \
-#        " FROM bioentry_dbxref JOIN dbxref USING (dbxref_id)" \
-#        " WHERE bioentry_id = %s" \
-#        " ORDER BY rank", (primary_id,))
-#    dbxrefs = []
-#    for dbname, accession, version in refs:
-#        if version and version != "0":
-#            v = "%s.%s" % (accession, version)
-#        else:
-#            v = accession
-#        dbxrefs.append((dbname, v))
-#    return {'cross_references': dbxrefs}
-
 def _retrieve_taxon(adaptor, primary_id, taxon_id):
     a = {}
     common_names = adaptor.execute_and_fetch_col0(
@@ -314,17 +353,35 @@ def _retrieve_taxon(adaptor, primary_id, taxon_id):
     if ncbi_taxids and ncbi_taxids[0] and ncbi_taxids[0] != "0":
         a['ncbi_taxid'] = ncbi_taxids[0]
 
-    # XXX 'no rank' or not?
-    # If we keep them, perhaps add " AND parent.parent_taxon_id <> 1"
-    taxonomy = adaptor.execute_and_fetch_col0(
-        "SELECT taxon_name.name" \
-        " FROM taxon t, taxon parent, taxon_name" \
-        " WHERE parent.taxon_id=taxon_name.taxon_id" \
-        " AND t.left_value BETWEEN parent.left_value AND parent.right_value" \
-        " AND name_class='scientific name'" \
-        " AND parent.node_rank <> 'no rank'" \
-        " AND t.taxon_id = %s" \
-        " ORDER BY parent.left_value", (taxon_id,))
+    #Old code used the left/right values in the taxon table to get the
+    #taxonomy lineage in one SQL command.  This was actually very slow,
+    #and would fail if the (optional) left/right values were missing.
+    #
+    #The following code is based on a contribution from Eric Gibert, and
+    #relies on the taxon table's parent_taxon_id field only (ignoring the
+    #optional left/right values).  This means that it has to make a
+    #separate SQL query for each entry in the lineage, but it does still
+    #appear to be *much* faster.  See Bug 2494. 
+    taxonomy = []
+    while taxon_id :
+        name, rank, parent_taxon_id = adaptor.execute_one(
+        "SELECT taxon_name.name, taxon.node_rank, taxon.parent_taxon_id" \
+        " FROM taxon, taxon_name" \
+        " WHERE taxon.taxon_id=taxon_name.taxon_id" \
+        " AND taxon_name.name_class='scientific name'" \
+        " AND taxon.taxon_id = %s", (taxon_id,))
+        if taxon_id == parent_taxon_id :
+            # If the taxon table has been populated by the BioSQL script
+            # load_ncbi_taxonomy.pl this is how top parent nodes are stored.
+            # Personally, I would have used a NULL parent_taxon_id here.
+            break
+        if rank != "no rank" :
+            #For consistency with older versions of Biopython, we are only
+            #interested in taxonomy entries with a stated rank.
+            #Add this to the start of the lineage list.
+            taxonomy.insert(0, name)
+        taxon_id = parent_taxon_id
+
     if taxonomy:
         a['taxonomy'] = taxonomy
     return a
