@@ -1,5 +1,6 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
-# Revisions 2007-2008 by Peter Cock.
+# Revisions 2007-2008 copyright by Peter Cock.  All rights reserved.
+# Revisions 2008 copyright by Cymon J. Cox.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -20,18 +21,27 @@ from time import gmtime, strftime
 # biopython
 from Bio import Alphabet
 from Bio.SeqUtils.CheckSum import crc64
-
+from Bio import Entrez
 
 class DatabaseLoader:
-    """Load a database with biopython objects.
-    """
-    def __init__(self, adaptor, dbid):
+    """Object used to load SeqRecord objects into a BioSQL database."""
+    def __init__(self, adaptor, dbid, fetch_NCBI_taxonomy=False):
         """Initialize with connection information for the database.
 
-        XXX Figure out what I need to load a database and document it.
+        Creating a DatabaseLoader object is normally handled via the
+        BioSeqDatabase DBServer object, for example:
+
+        from BioSQL import BioSeqDatabase
+        server = BioSeqDatabase.open_database(driver="MySQLdb", user="gbrowse",
+                         passwd = "biosql", host = "localhost", db="test_biosql")
+        try :
+            db = server["test"]
+        except KeyError :
+            db = server.new_database("test", description="For testing GBrowse")
         """
         self.adaptor = adaptor
         self.dbid = dbid
+        self.fetch_NCBI_taxonomy = fetch_NCBI_taxonomy
     
     def load_seqrecord(self, record):
         """Load a Biopython SeqRecord into the database.
@@ -50,6 +60,14 @@ class DatabaseLoader:
             self._load_seqfeature(seq_feature, seq_feature_num, bioentry_id)
 
     def _get_ontology_id(self, name, definition=None):
+        """Returns the identifier for the named ontology (PRIVATE).
+
+        This looks through the onotology table for a the given entry name.
+        If it is not found, a row is added for this ontology (using the
+        definition if supplied).  In either case, the id corresponding to
+        the provided name is returned, so that you can reference it in
+        another table.
+        """
         oids = self.adaptor.execute_and_fetch_col0(
             "SELECT ontology_id FROM ontology WHERE name = %s",
             (name,))
@@ -66,7 +84,7 @@ class DatabaseLoader:
                      ontology_id=None,
                      definition=None,
                      identifier=None):
-        """Get the id that corresponds to a term.
+        """Get the id that corresponds to a term (PRIVATE).
 
         This looks through the term table for a the given term. If it
         is not found, a new id corresponding to this term is created.
@@ -99,7 +117,7 @@ class DatabaseLoader:
             return self.adaptor.last_id("term")
 
     def _add_dbxref(self, dbname, accession, version):
-       """Insert a dbxref and return its id"""
+       """Insert a dbxref and return its id."""
        
        self.adaptor.execute(
            "INSERT INTO dbxref(dbname, accession, version)" \
@@ -107,7 +125,7 @@ class DatabaseLoader:
        return self.adaptor.last_id("dbxref")
            
     def _get_taxon_id(self, record):
-        """Get the taxon id for this record.
+        """Get the taxon id for this record (PRIVATE).
 
         record - a SeqRecord object
 
@@ -117,10 +135,11 @@ class DatabaseLoader:
         
         If the species isn't in the taxon table, and we have at
         least the NCBI taxon ID, scientific name or common name,
-        a minimal stub entry is created in the table.
+        at least a minimal stub entry is created in the table.
 
-        If this information is not in the record's annotation,
-        then None is returned.
+        Returns the taxon id (database key for the taxon table,
+        not an NCBI taxon ID), or None if the taxonomy information
+        is missing.
 
         See also the BioSQL script load_ncbi_taxonomy.pl which
         will populate and update the taxon/taxon_name tables
@@ -147,20 +166,6 @@ class DatabaseLoader:
                                 ncbi_taxon_id = int(db_xref[6:])
                                 break
                 if ncbi_taxon_id: break
-        if ncbi_taxon_id:
-            taxa = self.adaptor.execute_and_fetch_col0(
-                "SELECT taxon_id FROM taxon WHERE ncbi_taxon_id = %s",
-                (ncbi_taxon_id,))
-            if taxa:
-                #Good, we have mapped the NCBI taxid to a taxon table entry
-                return taxa[0]
-            #Bad, this NCBI taxon id is not in the database!
-
-        
-        # Either we didn't find the NCBI taxid in the record, or it
-        # isn't in the database yet. Next, we'll try and find a match
-        # based on the species name (stored in GenBank files as the
-        # the organism and/or the source).
 
         try :
             scientific_name = record.annotations["organism"][:255]
@@ -174,13 +179,23 @@ class DatabaseLoader:
         # Cropping it now should help in getting a match when searching,
         # and avoids an error if we try and add these to the database.
 
-        if ncbi_taxon_id is None \
-        and not common_name and not scientific_name :
+
+        if ncbi_taxon_id:
+            #Good, we have the NCBI taxon to go on - this is unambiguous :)
+            #Note that the scientific name and common name will only be
+            #used if we have to record a stub entry.
+            return self._get_taxon_id_from_ncbi_taxon_id(ncbi_taxon_id,
+                                                         scientific_name,
+                                                         common_name)
+        
+        if not common_name and not scientific_name :
             # Nothing to go on... and there is no point adding
             # a new entry to the database.  We'll just leave this
             # sequence's taxon as a NULL in the database.
             return None
 
+        # Next, we'll try to find a match based on the species name
+        # (stored in GenBank files as the organism and/or the source).
         if scientific_name:
             taxa = self.adaptor.execute_and_fetch_col0(
                 "SELECT taxon_id FROM taxon_name" \
@@ -208,40 +223,12 @@ class DatabaseLoader:
 
         # At this point, as far as we can tell, this species isn't
         # in the taxon table already.  So we'll have to add it.
-
-        """
-        # Possible simplification of the scary code below; pending
-        # discussion on the BioSQL mailing list...
+        # We don't have an NCBI taxonomy ID, so if we do record just
+        # a stub entry, there is no simple way to fix this later.
         #
-        # We will record the bare minimum; as long as the NCBI taxon
-        # id is present, then (re)running load_ncbi_taxonomy.pl should
-        # fill in the taxonomomy lineage.
+        # TODO - Should we try searching the NCBI taxonomy using the
+        # species name?
         #
-        # I am NOT going to try and record the lineage, even if it
-        # is in the record annotation as a list of names, as we won't
-        # know the NCBI taxon IDs for these parent nodes.
-        self.adaptor.execute(
-            "INSERT INTO taxon(parent_taxon_id, ncbi_taxon_id, node_rank,"\
-            " left_value, right_value)" \
-            " VALUES (%s, %s, %s, %s, %s)", (None,
-                                             ncbi_taxon_id,
-                                             "species",
-                                             None,
-                                             None))
-        taxon_id = self.adaptor.last_id("taxon")
-        if scientific_name:
-            self.adaptor.execute(
-                "INSERT INTO taxon_name(taxon_id, name, name_class)" \
-                "VALUES (%s, %s, 'scientific name')", (
-                taxon_id, scientific_name))
-        if common_name:
-            self.adaptor.execute(
-                "INSERT INTO taxon_name(taxon_id, name, name_class)" \
-                "VALUES (%s, %s, 'common name')", (
-                taxon_id, common_name))
-        return taxon_id
-        """
-
         # OK, let's try inserting the species.
         # Chances are we don't have enough information ...
         # Furthermore, it won't be in the hierarchy.
@@ -304,26 +291,239 @@ class DatabaseLoader:
 
         return taxon_id
 
+    def _fix_name_class(self, entrez_name) :
+        """Map Entrez name terms to those used in taxdump (PRIVATE).
+
+        We need to make this conversion to match the taxon_name.name_class
+        values used by the BioSQL load_ncbi_taxonomy.pl script.
+        
+        e.g.
+        "ScientificName" -> "scientific name",
+        "EquivalentName" -> "equivalent name",
+        "Synonym" -> "synonym",
+        """
+        #Add any special cases here:
+        #
+        #known = {}
+        #try :
+        #    return known[entrez_name]
+        #except KeyError:
+        #    pass
+
+        #Try automatically by adding spaces before each capital
+        def add_space(letter) :
+            if letter.isupper() :
+                return " "+letter.lower()
+            else :
+                return letter
+        answer = "".join([add_space(letter) for letter in entrez_name]).strip()
+        assert answer == answer.lower()
+        return answer
+
+    def _get_taxon_id_from_ncbi_taxon_id(self, ncbi_taxon_id,
+                                         scientific_name = None,
+                                         common_name = None):
+        """Get the taxon id for this record from the NCBI taxon ID (PRIVATE).
+
+        ncbi_taxon_id - string containing an NCBI taxon id
+        scientific_name - string, used if a stub entry is recorded
+        common_name - string, used if a stub entry is recorded
+        
+        This searches the taxon table using ONLY the NCBI taxon ID
+        to find the matching taxon table entry's ID (database key).
+        
+        If the species isn't in the taxon table, and the fetch_NCBI_taxonomy
+        flag is true, Biopython will attempt to go online using Bio.Entrez
+        to fetch the official NCBI lineage, recursing up the tree until an
+        existing entry is found in the database or the full lineage has been
+        fetched.
+
+        Otherwise the NCBI taxon ID, scientific name and common name are
+        recorded as a minimal stub entry in the taxon and taxon_name tables.
+        Any partial information about the lineage from the SeqRecord is NOT
+        recorded.  This should mean that (re)running the BioSQL script
+        load_ncbi_taxonomy.pl can fill in the taxonomy lineage.
+
+        Returns the taxon id (database key for the taxon table, not
+        an NCBI taxon ID).
+        """
+        assert ncbi_taxon_id
+
+        taxon_id = self.adaptor.execute_and_fetch_col0(
+            "SELECT taxon_id FROM taxon WHERE ncbi_taxon_id = %s",
+            (ncbi_taxon_id,))
+        if taxon_id:
+            #Good, we have mapped the NCBI taxid to a taxon table entry
+            return taxon_id[0]
+
+        # At this point, as far as we can tell, this species isn't
+        # in the taxon table already.  So we'll have to add it.
+
+        parent_taxon_id = None
+        rank = "species"
+        genetic_code = None
+        mito_genetic_code = None
+        species_names = []
+        if scientific_name :
+            species_names.append(("scientific name", scientific_name))
+        if common_name :
+            species_names.append(("common name", common_name))
+        
+        if self.fetch_NCBI_taxonomy :
+            #Go online to get the parent taxon ID!
+            handle = Entrez.efetch(db="taxonomy",id=ncbi_taxon_id,retmode="XML")
+            taxonomic_record = Entrez.read(handle)
+            if len(taxonomic_record) == 1:
+                assert taxonomic_record[0]["TaxId"] == str(ncbi_taxon_id), \
+                       "%s versus %s" % (taxonomic_record[0]["TaxId"],
+                                         ncbi_taxon_id)
+                parent_taxon_id = self._get_taxon_id_from_ncbi_lineage( \
+                                            taxonomic_record[0]["LineageEx"])
+                rank = taxonomic_record[0]["Rank"]
+                genetic_code = taxonomic_record[0]["GeneticCode"]["GCId"]
+                mito_genetic_code = taxonomic_record[0]["MitoGeneticCode"]["MGCId"]
+                species_names = [("scientific name",
+                                  taxonomic_record[0]["ScientificName"])]
+                try :
+                    for name_class, names in taxonomic_record[0]["OtherNames"].iteritems():
+                        name_class = self._fix_name_class(name_class)
+                        if not isinstance(names, list) :
+                            #The Entrez parser seems to return single entry
+                            #lists as just a string which is annoying.
+                            names = [names]
+                        for name in names :
+                            #Want to ignore complex things like ClassCDE entries
+                            if isinstance(name, basestring) :
+                                species_names.append((name_class, name))
+                except KeyError :
+                    #OtherNames isn't always present,
+                    #e.g. NCBI taxon 41205, Bromheadia finlaysoniana
+                    pass
+        else :
+            pass
+            # If we are not allowed to go online, we will record the bare minimum;
+            # as long as the NCBI taxon id is present, then (re)running
+            # load_ncbi_taxonomy.pl should fill in the taxonomomy lineage
+            # (and update the species names).
+            #
+            # I am NOT going to try and record the lineage, even if it
+            # is in the record annotation as a list of names, as we won't
+            # know the NCBI taxon IDs for these parent nodes.
+
+        self.adaptor.execute(
+            "INSERT INTO taxon(parent_taxon_id, ncbi_taxon_id, node_rank,"\
+            " genetic_code, mito_genetic_code, left_value, right_value)" \
+            " VALUES (%s, %s, %s, %s, %s, %s, %s)", (parent_taxon_id,
+                                                     ncbi_taxon_id,
+                                                     rank,
+                                                     genetic_code,
+                                                     mito_genetic_code,
+                                                     None,
+                                                     None))
+        taxon_id = self.adaptor.last_id("taxon")
+
+        #Record the scientific name, common name, etc
+        for name_class, name in species_names :
+            self.adaptor.execute(
+                "INSERT INTO taxon_name(taxon_id, name, name_class)" \
+                " VALUES (%s, %s, %s)", (taxon_id, 
+                                         name[:255], 
+                                         name_class))
+        return taxon_id
+
+    def _get_taxon_id_from_ncbi_lineage(self, taxonomic_lineage) :
+        """This is recursive! (PRIVATE).
+
+        taxonomic_lineage - list of taxonomy dictionaries from Bio.Entrez
+
+        First dictionary in list is the taxonomy root, highest would be the species.
+        Each dictionary includes:
+        - TaxID (string, NCBI taxon id)
+        - Rank (string, e.g. "species", "genus", ..., "phylum", ...)
+        - ScientificName (string)
+        (and that is all at the time of writing)
+
+        This method will record all the lineage given, returning the the taxon id
+        (database key, not NCBI taxon id) of the final entry (the species).
+        """
+        ncbi_taxon_id = taxonomic_lineage[-1]["TaxId"]
+
+        #Is this in the database already?  Check the taxon table...
+        taxon_id = self.adaptor.execute_and_fetch_col0(
+            "SELECT taxon_id FROM taxon" \
+            " WHERE ncbi_taxon_id=%s" % ncbi_taxon_id)
+        if taxon_id:
+            # we could verify that the Scientific Name etc in the database
+            # is the same and update it or print a warning if not...
+            if isinstance(taxon_id, list) :
+                assert len(taxon_id)==1
+                return taxon_id[0]
+            else :
+                return taxon_id
+
+        #We have to record this.
+        if len(taxonomic_lineage) > 1 :
+            #Use recursion to find out the taxon id (database key) of the parent.
+            parent_taxon_id = self._get_taxon_id_from_ncbi_lineage(taxonomic_lineage[:-1])
+            assert isinstance(parent_taxon_id, int) or isinstance(parent_taxon_id, long), repr(parent_taxon_id)
+        else :
+            parent_taxon_id = None
+
+        # INSERT new taxon
+        rank = taxonomic_lineage[-1].get("Rank", None)
+        self.adaptor.execute(
+                "INSERT INTO taxon(ncbi_taxon_id, parent_taxon_id, node_rank)"\
+                " VALUES (%s, %s, %s)", (ncbi_taxon_id, parent_taxon_id, rank))
+        taxon_id = self.adaptor.last_id("taxon")
+        assert isinstance(taxon_id, int) or isinstance(taxon_id, long), repr(taxon_id)
+        # ... and its name in taxon_name
+        scientific_name = taxonomic_lineage[-1].get("ScientificName", None)
+        if scientific_name :
+            self.adaptor.execute(
+                    "INSERT INTO taxon_name(taxon_id, name, name_class)" \
+                    " VALUES (%s, %s, 'scientific name')", (taxon_id, 
+                                                            scientific_name[:255]))
+        return taxon_id
+
+
     def _load_bioentry_table(self, record):
-        """Fill the bioentry table with sequence information.
+        """Fill the bioentry table with sequence information (PRIVATE).
+
+        record - SeqRecord object to add to the database.
         """
         # get the pertinent info and insert it
         
-        if record.id.find('.') >= 0: # try to get a version from the id
+        if record.id.count(".") == 1: # try to get a version from the id
+            #This assumes the string is something like "XXXXXXXX.123"
             accession, version = record.id.split('.')
-            version = int(version)
+            try :
+                version = int(version)
+            except ValueError :
+                accession = record.id
+                version = 0
         else: # otherwise just use a version of 0
             accession = record.id
             version = 0
 
+        if "accessions" in record.annotations \
+        and isinstance(record.annotations["accessions"],list) \
+        and record.annotations["accessions"] :
+            #Take the first accession (one if there is more than one)
+            accession = record.annotations["accessions"][0]
+
         #Find the taxon id (this is not just the NCBI Taxon ID)
         #NOTE - If the species isn't defined in the taxon table,
         #a new minimal entry is created.
         taxon_id = self._get_taxon_id(record)
 
-        identifier = record.annotations.get('gi')
+        if "gi" in record.annotations :
+            identifier = record.annotations["gi"]
+        else :
+            identifier = record.id
+
+        #Allow description and division to default to NULL as in BioPerl.
         description = getattr(record, 'description', None)
-        division = record.annotations.get("data_file_division", "UNK")
+        division = record.annotations.get("data_file_division", None)
         
         sql = """
         INSERT INTO bioentry (
@@ -344,6 +544,8 @@ class DatabaseLoader:
          %s,
          %s,
          %s)"""
+        #print self.dbid, taxon_id, record.name, accession, identifier, \
+        #        division, description, version
         self.adaptor.execute(sql, (self.dbid,
                                    taxon_id,
                                    record.name, 
@@ -359,6 +561,9 @@ class DatabaseLoader:
 
     def _load_bioentry_date(self, record, bioentry_id):
         """Add the effective date of the entry into the database.
+
+        record - a SeqRecord object with an annotated date
+        bioentry_id - corresponding database identifier
         """
         # dates are GenBank style, like:
         # 14-SEP-2000
@@ -372,7 +577,10 @@ class DatabaseLoader:
         self.adaptor.execute(sql, (bioentry_id, date_id, date))
 
     def _load_biosequence(self, record, bioentry_id):
-        """Load the biosequence table in the database.
+        """Record a SeqRecord's sequence and alphabet in the database (PRIVATE).
+
+        record - a SeqRecord object with a seq property
+        bioentry_id - corresponding database identifier
         """
         # determine the string representation of the alphabet
         if isinstance(record.seq.alphabet, Alphabet.DNAAlphabet):
@@ -393,6 +601,11 @@ class DatabaseLoader:
                                    alphabet))
 
     def _load_comment(self, record, bioentry_id):
+        """Record a SeqRecord's annotated comment in the database (PRIVATE).
+
+        record - a SeqRecord object with an annotated comment
+        bioentry_id - corresponding database identifier
+        """
         # Assume annotations['comment'] is not a list
         comment = record.annotations.get('comment')
         if not comment:
@@ -404,7 +617,15 @@ class DatabaseLoader:
         self.adaptor.execute(sql, (bioentry_id, comment, 1))
         
     def _load_annotations(self, record, bioentry_id) :
-        """Record misc annotations in the bioentry_qualifier_value table"""
+        """Record a SeqRecord's misc annotations in the database (PRIVATE).
+
+        The annotation strings are recorded in the bioentry_qualifier_value
+        table, except for special cases like the reference, comment and
+        taxonomy which are handled with their own tables.
+
+        record - a SeqRecord object with an annotations dictionary
+        bioentry_id - corresponding database identifier
+        """
         mono_sql = "INSERT INTO bioentry_qualifier_value" \
                    "(bioentry_id, term_id, value)" \
                    " VALUES (%s, %s, %s)"
@@ -440,6 +661,11 @@ class DatabaseLoader:
 
 
     def _load_reference(self, reference, rank, bioentry_id):
+        """Record a SeqRecord's annotated references in the database (PRIVATE).
+
+        record - a SeqRecord object with annotated references
+        bioentry_id - corresponding database identifier
+        """
 
         refs = None
         if reference.medline_id:
@@ -500,7 +726,7 @@ class DatabaseLoader:
                                    start, end, rank + 1))
         
     def _load_seqfeature(self, feature, feature_rank, bioentry_id):
-        """Load a biopython SeqFeature into the database.
+        """Load a biopython SeqFeature into the database (PRIVATE).
         """
         seqfeature_id = self._load_seqfeature_basic(feature.type, feature_rank,
                                                     bioentry_id)
@@ -508,7 +734,7 @@ class DatabaseLoader:
         self._load_seqfeature_qualifiers(feature.qualifiers, seqfeature_id)
 
     def _load_seqfeature_basic(self, feature_type, feature_rank, bioentry_id):
-        """Load the first tables of a seqfeature and returns the id.
+        """Load the first tables of a seqfeature and returns the id (PRIVATE).
 
         This loads the "key" of the seqfeature (ie. CDS, gene) and
         the basic seqfeature table itself.
@@ -516,7 +742,6 @@ class DatabaseLoader:
         ontology_id = self._get_ontology_id('SeqFeature Keys')
         seqfeature_key_id = self._get_term_id(feature_type,
                                               ontology_id = ontology_id)
-        
         # XXX source is always EMBL/GenBank/SwissProt here; it should depend on
         # the record (how?)
         source_cat_id = self._get_ontology_id('SeqFeature Sources')
@@ -532,7 +757,7 @@ class DatabaseLoader:
         return seqfeature_id
 
     def _load_seqfeature_locations(self, feature, seqfeature_id):
-        """Load all of the locations for a SeqFeature into tables.
+        """Load all of the locations for a SeqFeature into tables (PRIVATE).
 
         This adds the locations related to the SeqFeature into the
         seqfeature_location table. Fuzzies are not handled right now.
@@ -545,6 +770,9 @@ class DatabaseLoader:
             start = 3, end = 4, rank = 2
             start = 5, end = 6, rank = 3
         """
+        # TODO - Record an ontology for the locations (using location.term_id)
+        # which for now as in BioPerl we leave defaulting to NULL.
+        
         # two cases, a simple location or a split location
         if not feature.sub_features:    # simple location
             self._insert_seqfeature_location(feature, 1, seqfeature_id)
@@ -555,26 +783,61 @@ class DatabaseLoader:
                                                  seqfeature_id)
 
     def _insert_seqfeature_location(self, feature, rank, seqfeature_id):
-        """Add a location of a SeqFeature to the seqfeature_location table.
-        """
-        sql = r"INSERT INTO location (seqfeature_id, " \
-              r"start_pos, end_pos, strand, rank) " \
-              r"VALUES (%s, %s, %s, %s, %s)"
+        """Add a location of a SeqFeature to the seqfeature_location table (PRIVATE).
 
+        TODO - Add location_operators to location_qualifier_value.
+        """
         # convert biopython locations to the 1-based location system
         # used in bioSQL
         # XXX This could also handle fuzzies
         start = feature.location.nofuzzy_start + 1
         end = feature.location.nofuzzy_end
+
         # Biopython uses None when we don't know strand information but
         # BioSQL requires something (non null) and sets this as zero
         # So we'll use the strand or 0 if Biopython spits out None
         strand = feature.strand or 0
 
-        self.adaptor.execute(sql, (seqfeature_id, start, end, strand, rank))
+        # TODO - Record an ontology term for the location (location.term_id)
+        # which for now like BioPerl we'll leave as NULL.
+        loc_term_id = None
+
+        if feature.ref:
+            # sub_feature remote locations when they are in the same db as the current
+            # record do not have a value for ref_db, which the SeqFeature object
+            # stores as None. BioSQL schema requires a varchar and is not NULL 
+            dbxref_id = self._get_dbxref_id(feature.ref_db or "", feature.ref)
+        else :
+            dbxref_id = None
+
+        sql = r"INSERT INTO location (seqfeature_id, dbxref_id, term_id," \
+              r"start_pos, end_pos, strand, rank) " \
+              r"VALUES (%s, %s, %s, %s, %s, %s, %s)"
+        self.adaptor.execute(sql, (seqfeature_id, dbxref_id, loc_term_id,
+                                   start, end, strand, rank))
+
+        """
+        # See Bug 2677
+        # TODO - Record the location_operator (e.g. "join" or "order")
+        # using the location_qualifier_value table (which we and BioPerl
+        # have historically left empty).
+        # Note this will need an ontology term for the location qualifer
+        # (location_qualifier_value.term_id) for which oddly the schema
+        # does not allow NULL.
+        if feature.location_operator:
+            #e.g. "join" (common),
+            #or "order" (see Tests/GenBank/protein_refseq2.gb)
+            location_id = self.adaptor.last_id('location')
+            loc_qual_term_id = None # Not allowed in BioSQL v1.0.1
+            sql = r"INSERT INTO location_qualifier_value" \
+                  r"(location_id, term_id, value)" \
+                  r"VALUES (%s, %s, %s)"
+            self.adaptor.execute(sql, (location_id, loc_qual_term_id,
+                                       feature.location_operator))
+        """
 
     def _load_seqfeature_qualifiers(self, qualifiers, seqfeature_id):
-        """Insert the (key, value) pair qualifiers relating to a feature.
+        """Insert the (key, value) pair qualifiers relating to a feature (PRIVATE).
 
         Qualifiers should be a dictionary of the form:
             {key : [value1, value2]}
@@ -589,8 +852,13 @@ class DatabaseLoader:
                 qualifier_key_id = self._get_term_id(qualifier_key,
                                                   ontology_id=tag_ontology_id)
                 # now add all of the values to their table
-                for qual_value_rank in range(len(qualifiers[qualifier_key])):
-                    qualifier_value = qualifiers[qualifier_key][qual_value_rank]
+                entries = qualifiers[qualifier_key]
+                if not isinstance(entries, list) :
+                    # Could be a plain string, or an int or a float.
+                    # However, we exect a list of strings here.
+                    entries = [entries]
+                for qual_value_rank in range(len(entries)):
+                    qualifier_value = entries[qual_value_rank]
                     sql = r"INSERT INTO seqfeature_qualifier_value "\
                           r" (seqfeature_id, term_id, rank, value) VALUES"\
                           r" (%s, %s, %s, %s)"
@@ -608,7 +876,7 @@ class DatabaseLoader:
 
 
     def _load_seqfeature_dbxref(self, dbxrefs, seqfeature_id):
-        """ _load_seqfeature_dbxref(self, dbxrefs, seqfeature_id)
+        """Add database crossreferences of a SeqFeature to the database (PRIVATE).
 
             o dbxrefs           List, dbxref data from the source file in the
                                 format <database>:<accession>
@@ -673,8 +941,8 @@ class DatabaseLoader:
 
         """
         # Check for an existing record
-        sql = r'SELECT seqfeature_id, dbxref_id FROM seqfeature_dbxref ' \
-              r'WHERE seqfeature_id = "%s" AND dbxref_id = "%s"'
+        sql = r"SELECT seqfeature_id, dbxref_id FROM seqfeature_dbxref " \
+              r"WHERE seqfeature_id = '%s' AND dbxref_id = '%s'"
         result = self.adaptor.execute_and_fetch_col0(sql, (seqfeature_id,
                                                            dbxref_id))
         # If there was a record, return without executing anything, else create
@@ -694,9 +962,9 @@ class DatabaseLoader:
         return (seqfeature_id, dbxref_id)
 
     def _load_dbxrefs(self, record, bioentry_id) :
-        """Load any sequence level cross references into the database.
+        """Load any sequence level cross references into the database (PRIVATE).
 
-        See table bioentry_dbxref"""
+        See table bioentry_dbxref."""
         for rank, value in enumerate(record.dbxrefs):
             # Split the DB:accession string at first colon.
             # We have to cope with things like:
@@ -724,8 +992,8 @@ class DatabaseLoader:
 
         """
         # Check for an existing record
-        sql = r'SELECT bioentry_id, dbxref_id FROM bioentry_dbxref ' \
-              r'WHERE bioentry_id = "%s" AND dbxref_id = "%s"'
+        sql = r"SELECT bioentry_id, dbxref_id FROM bioentry_dbxref " \
+              r"WHERE bioentry_id = '%s' AND dbxref_id = '%s'"
         result = self.adaptor.execute_and_fetch_col0(sql, (bioentry_id,
                                                            dbxref_id))
         # If there was a record, return without executing anything, else create
@@ -769,4 +1037,3 @@ class DatabaseRemover:
         sql = r"DELETE FROM biodatabase WHERE biodatabase_id = %s"
         self.adaptor.execute(sql, (self.dbid,))
 
-        
