@@ -18,7 +18,7 @@ gb_file_dir = os.path.join(os.getcwd(), 'GenBank')
 test_files = ['noref.gb', 'cor6_6.gb', 'iro.gb', 'pri1.gb', 'arab1.gb',
               'protein_refseq.gb', 'extra_keywords.gb', 'one_of.gb',
               'NT_019265.gb', 'origin_line.gb', 'blank_seq.gb',
-              'dbsource_wrap.gb', 'gbvrl1_start.seq']
+              'dbsource_wrap.gb', 'gbvrl1_start.seq', 'NC_005816.gb']
 
 # We only test writing on a subset of the examples:
 write_format_files = ['noref.gb', 'cor6_6.gb', 'iro.gb', 'pri1.gb', 'arab1.gb',
@@ -59,11 +59,12 @@ for parser in all_parsers:
         while 1:
             cur_record = iterator.next()
 
-            if not(cur_record):
+            if cur_record is None:
                 break
 
             if isinstance(parser, GenBank.FeatureParser):
-                print "***Record from the FeatureParser"
+                print "***Record from %s with the FeatureParser" \
+                      % filename.split(os.path.sep)[-1]
                 print "Seq:", repr(cur_record.seq)
                 print "Id:", cur_record.id
                 print "Name:", cur_record.name
@@ -83,8 +84,11 @@ for parser in all_parsers:
                 print "Feaures"
                 for feature in cur_record.features:
                     print feature
+                print "DB cross refs", cur_record.dbxrefs
             elif isinstance(parser, GenBank.RecordParser):
-                print "***Record from the RecordParser"
+                print "***Record from %s with the RecordParser" \
+                      % filename.split(os.path.sep)[-1]
+                print "sequence length: %i" % len(cur_record.sequence)
                 print "locus:", cur_record.locus
                 print "definition:", cur_record.definition
                 print "accession:", cur_record.accession
@@ -122,13 +126,13 @@ def do_comparison(good_record, test_record):
 
         if not(good_line) and not(test_line):
             break
-
         if not(good_line):
             raise AssertionError("Extra info in Test: `%s`" % test_line)
         if not(test_line):
             raise AssertionError("Extra info in Expected: `%s`" % good_line)
-
-        assert test_line == good_line, \
+        test_normalized = ' '.join([x for x in test_line.split() if x])
+        good_normalized = ' '.join([x for x in good_line.split() if x])
+        assert test_normalized == good_normalized, \
                "Expected does not match Test.\nExpect:`%s`\nTest  :`%s`\n" % \
                (good_line, test_line)
     
