@@ -3,6 +3,9 @@
 Goals:
     Make sure that all retrieval is working as expected.
 """
+import sys
+if sys.modules.has_key('requires_internet'):
+    del sys.modules['requires_internet']
 import requires_internet
 
 import os
