@@ -1,18 +1,40 @@
-# Copyright 2007 by Peter Cock.  All rights reserved.
+# Copyright 2007-2008 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
 import os
-from sets import Set
+
+#TODO - Remove this work around once we drop python 2.3 support
+try:
+    set = set
+except NameError:
+    from sets import Set as set
+
 from Bio import SeqIO
+from Bio import AlignIO
 from Bio.SeqRecord import SeqRecord
 from Bio.Seq import Seq
 from StringIO import StringIO
-from Bio.Alphabet import generic_protein, generic_rna, generic_dna
+from Bio import Alphabet
 
-#Longer list including alignment only file formats we can read AND write:
-test_write_read_alignment_formats = SeqIO._FormatToWriter.keys()[:]
+protein_alphas = [Alphabet.generic_protein]
+dna_alphas = [Alphabet.generic_dna]
+rna_alphas = [Alphabet.generic_rna]
+nucleotide_alphas = [Alphabet.generic_nucleotide,
+                     Alphabet.Gapped(Alphabet.generic_nucleotide)]
+no_alpha_formats = ["fasta","clustal","phylip","tab","ig","stockholm","emboss"]
+
+#List of formats including alignment only file formats we can read AND write.
+#The list is initially hard coded to preserve the original order of the unit
+#test output, with any new formats added since appended to the end.
+test_write_read_alignment_formats = ["fasta","clustal","phylip","stockholm"]
+for format in SeqIO._FormatToWriter :
+    if format not in test_write_read_alignment_formats :
+        test_write_read_alignment_formats.append(format)
+for format in AlignIO._FormatToWriter :
+    if format not in test_write_read_alignment_formats :
+        test_write_read_alignment_formats.append(format)
 
 # test_files is a list of tuples containing:
 # - string:  file format
@@ -24,6 +46,8 @@ test_files = [ \
 #Following examples are also used in test_Clustalw.py
     ("clustal",True,  'Clustalw/cw02.aln', 2),
     ("clustal",True,  'Clustalw/opuntia.aln', 7),
+    ("clustal",True,  'Clustalw/hedgehog.aln', 5),
+    ("clustal",True,  'Clustalw/odd_consensus.aln', 2),
 #Following nucleic examples are also used in test_Fasta2.py
     ("fasta",  False, 'Nucleic/lupine.nu', 1),
     ("fasta",  False, 'Nucleic/elderberry.nu', 1),
@@ -83,6 +107,7 @@ test_files = [ \
     ("genbank",False, 'GenBank/origin_line.gb', 1),
     ("genbank",False, 'GenBank/blank_seq.gb', 1),
     ("genbank",False, 'GenBank/dbsource_wrap.gb', 1),
+    ("genbank",False, 'GenBank/NC_005816.gb', 1),
 # The next example is a truncated copy of gbvrl1.seq from
 # ftp://ftp.ncbi.nih.gov/genbank/gbvrl1.seq.gz
 # This includes an NCBI header, and the first three records:
@@ -90,49 +115,74 @@ test_files = [ \
 #Following files are also used in test_GFF.py
     ("genbank",False, 'GFF/NC_001422.gbk', 1),
 #Following files are currently only used here:
-    ("embl",      False, 'EMBL/TRBG361.embl', 1),
-    ("embl",      False, 'EMBL/DD231055_edited.embl', 1),
-    ("embl",      False, 'EMBL/SC10H5.embl', 1), # Pre 2006 style ID line
-    ("embl",      False, 'EMBL/U87107.embl', 1), # Old ID line with SV line
+    ("embl",   False, 'EMBL/TRBG361.embl', 1),
+    ("embl",   False, 'EMBL/DD231055_edited.embl', 1),
+    ("embl",   False, 'EMBL/SC10H5.embl', 1), # Pre 2006 style ID line
+    ("embl",   False, 'EMBL/U87107.embl', 1), # Old ID line with SV line
+    ("embl",   False, 'EMBL/AAA03323.embl', 1), # 2008, PA line but no AC
     ("stockholm", True,  'Stockholm/simple.sth', 2),
     ("stockholm", True,  'Stockholm/funny.sth', 5),
-#Following PHYLIP files are currently only used here (test_SeqIO)
+#Following PHYLIP files are currently only used here and in test_AlignIO.py,
 #and are mostly from Joseph Felsenstein's PHYLIP v3.6 documentation:
-    ("phylip",    True,  'Phylip/reference_dna.phy', 6),
-    ("phylip",    True,  'Phylip/reference_dna2.phy', 6),
-    ("phylip",    True,  'Phylip/hennigian.phy', 10),
-    ("phylip",    True,  'Phylip/horses.phy', 10),
-    ("phylip",    True,  'Phylip/random.phy', 10),
-    ("phylip",    True,  'Phylip/interlaced.phy', 3),
-    ("phylip",    True,  'Phylip/interlaced2.phy', 4),
+    ("phylip", True,  'Phylip/reference_dna.phy', 6),
+    ("phylip", True,  'Phylip/reference_dna2.phy', 6),
+    ("phylip", True,  'Phylip/hennigian.phy', 10),
+    ("phylip", True,  'Phylip/horses.phy', 10),
+    ("phylip", True,  'Phylip/random.phy', 10),
+    ("phylip", True,  'Phylip/interlaced.phy', 3),
+    ("phylip", True,  'Phylip/interlaced2.phy', 4),
+#Following are EMBOSS simple or pairs format alignments
+    ("emboss", True,  'Emboss/alignret.txt', 4),
+    ("emboss", False, 'Emboss/needle.txt', 10),
+    ("emboss", True,  'Emboss/water.txt', 2),
+#Following PHD (PHRAP) sequencing files are also used in test_Phd.py
+    ("phd", False, 'Phd/phd1', 3),
+    ("phd", False, 'Phd/phd2', 1),
+#Following ACE assembly files are also used in test_Phd.py
+    ("ace", False, 'Ace/contig1.ace', 2),
+    ("ace", False, 'Ace/consed_sample.ace', 1),
+    ("ace", False, 'Ace/seq.cap.ace', 1),
+#Following IntelliGenetics / MASE files are also used in test_intelligenetics.py
+    ("ig",  False, 'IntelliGenetics/TAT_mase_nuc.txt', 17),
+    ("ig",  True,  'IntelliGenetics/VIF_mase-pro.txt', 16),
+    #This next file is a MASE alignment but sequence O_ANT70 is shorter than
+    #the others (so the to_alignment() call will fail).  Perhaps MASE doesn't
+    #write trailing gaps?
+    ("ig",  False,  'IntelliGenetics/vpu_nucaligned.txt', 9),
+#Following NBRD-PIR files are used in test_nbrf.py
+    ("pir", False, 'NBRF/B_nuc.pir', 444),
+    ("pir", False, 'NBRF/Cw_prot.pir', 111),
+    ("pir", False, 'NBRF/DMA_nuc.pir', 4),
+    ("pir", False, 'NBRF/DMB_prot.pir', 6),
+    ("pir", True,  'NBRF/clustalw.pir', 2),
     ]
 
 # This is a list of two-tuples.  Each tuple contains a
 # list of SeqRecord objects and a description (string)
 test_records = [
     ([], "zero records"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",generic_protein), id="Alpha"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",generic_protein), id="Gamma"),
-      SeqRecord(Seq("DITHGVG",generic_protein), id="delta")],
+    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
+      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma"),
+      SeqRecord(Seq("DITHGVG",Alphabet.generic_protein), id="delta")],
      "three peptides of different lengths"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",generic_protein), id="Alpha"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",generic_protein), id="Beta"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",generic_protein), id="Gamma")],
+    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
+      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
+      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma")],
      "three proteins alignment"),
-    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",generic_dna), id="X"),
-      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",generic_dna), id="Y"),
-      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",generic_dna), id="Z")],
+    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",Alphabet.generic_dna), id="X"),
+      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",Alphabet.generic_dna), id="Y"),
+      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",Alphabet.generic_dna), id="Z")],
      "three DNA sequence alignment"),
-    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",generic_dna), id="X",
-                name="The\nMystery\rSequece:%sX" % os.linesep),
-      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",generic_dna), id="Y",
+    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",Alphabet.generic_dna), id="X",
+                name="The\nMystery\rSequece:\r\nX"),
+      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",Alphabet.generic_dna), id="Y",
                 description="an%sevil\rdescription right\nhere" % os.linesep),
-      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",generic_dna), id="Z")],
+      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",Alphabet.generic_dna), id="Z")],
      "3 DNA seq alignment with CR/LF in name/descr"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",generic_protein), id="Alpha"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",generic_protein), id="Beta"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",generic_protein), id="Beta"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",generic_protein), id="Gamma")],
+    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
+      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
+      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
+      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma")],
      "alignment with repeated record"),
     ]
 # Meddle with the annotation too:
@@ -148,13 +198,13 @@ test_records[4][0][2].annotations["weight"] = 2.5
 
 def records_match(record_one, record_two) :
     """This is meant to be a strict comparison for exact agreement"""
-    if record_one.id <> record_two.id :
+    if record_one.id != record_two.id :
         return False
-    if record_one.name <> record_two.name :
+    if record_one.name != record_two.name :
         return False
-    if record_one.description <> record_two.description :
+    if record_one.description != record_two.description :
         return False
-    if record_one.seq.tostring() <> record_two.seq.tostring() :
+    if record_one.seq.tostring() != record_two.seq.tostring() :
         return False
     #Close enough... should I check for features, annotation etc?
     return True
@@ -197,18 +247,21 @@ def alignment_summary(alignment, index=" ") :
 
 def check_simple_write_read(records, indent=" ") :
     #print indent+"Checking we can write and then read back these records"
-    for format in SeqIO._FormatToWriter.keys() :
+    for format in test_write_read_alignment_formats :
         print indent+"Checking can write/read as '%s' format" % format
         
         #Going to write to a handle...
         handle = StringIO()
         
         try :
-            SeqIO.write(sequences=records, handle=handle, format=format)
+            c = SeqIO.write(sequences=records, handle=handle, format=format)
+            assert c == len(records)
         except ValueError, e :
             #This is often expected to happen, for example when we try and
             #write sequences of different lengths to an alignment file.
             print indent+"Failed: %s" % str(e)
+            assert format != t_format, \
+                   "Should be able to re-write in the original format!"
             #Carry on to the next format:
             continue
 
@@ -226,14 +279,39 @@ def check_simple_write_read(records, indent=" ") :
                               % (str(e), repr(handle.read()), repr(records)))
 
         assert len(records2) == t_count
-        for i in range(t_count) :
+        for r1, r2 in zip(records, records2) :
             #Check the bare minimum (ID and sequence) as
             #many formats can't store more than that.
-            #
-            #Note some formats allow spaces, others don't.
-            #Assume spaces are turned into underscores.
-            records[i].id.replace(" ","_") == records2[i].id.replace(" ","_")
-            records[i].seq.data == records2[i].seq.data
+
+            #Check the sequence
+            if format == "genbank" :
+                #The GenBank parser will convert everything to upper case.
+                assert r1.seq.tostring().upper() == r2.seq.tostring()
+            else :
+                assert r1.seq.tostring() == r2.seq.tostring()
+            #Beware of different quirks and limitations in the
+            #valid character sets and the identifier lengths!
+            if format=="phylip" :
+                assert r1.id.replace("[","").replace("]","")[:10] == r2.id, \
+                       "'%s' vs '%s'" % (r1.id, r2.id)
+            elif format=="clustal" :
+                assert r1.id.replace(" ","_")[:30] == r2.id, \
+                       "'%s' vs '%s'" % (r1.id, r2.id)
+            elif format=="stockholm" :
+                assert r1.id.replace(" ","_") == r2.id, \
+                       "'%s' vs '%s'" % (r1.id, r2.id)
+            elif format=="fasta" :
+                assert r1.id.split()[0] == r2.id
+            else :
+                assert r1.id == r2.id, \
+                       "'%s' vs '%s'" % (r1.id, r2.id)
+
+
+#Check parsers can cope with an empty file
+for t_format in SeqIO._FormatToIterator :
+    handle = StringIO()
+    records = list(SeqIO.parse(handle, t_format))
+    assert len(records) == 0
 
 for (t_format, t_alignment, t_filename, t_count) in test_files :
     print "Testing reading %s format file %s" % (t_format, t_filename)
@@ -241,7 +319,8 @@ for (t_format, t_alignment, t_filename, t_count) in test_files :
 
     #Try as an iterator using handle
     records  = list(SeqIO.parse(handle=open(t_filename,"r"), format=t_format))
-    assert len(records)  == t_count
+    assert len(records)  == t_count, \
+         "Found %i records but expected %i" % (len(records), t_count)
 
     #Try using the iterator with a for loop
     records2 = []
@@ -257,7 +336,10 @@ for (t_format, t_alignment, t_filename, t_count) in test_files :
             record = seq_iterator.next()
         except StopIteration :
             record = None
-        if record :
+        #Note that if the SeqRecord class has a __len__ method,
+        #and it has a zero-length sequence, this would fail an
+        #"if record" test.
+        if record is not None :
             records3.append(record)
         else :
             break
@@ -298,7 +380,7 @@ for (t_format, t_alignment, t_filename, t_count) in test_files :
         assert isinstance(record.id, basestring)
         assert isinstance(record.name, basestring)
         assert isinstance(record.description, basestring)
-        assert record.id <> ""
+        assert record.id != ""
 
         if "accessions" in record.annotations :
             accs = record.annotations["accessions"]
@@ -306,7 +388,7 @@ for (t_format, t_alignment, t_filename, t_count) in test_files :
             for acc in accs :
                 assert acc and acc == acc.strip(), \
                     "Bad accession in annotations: %s" % repr(acc)
-            assert len(Set(accs)) == len(accs), \
+            assert len(set(accs)) == len(accs), \
                    "Repeated accession in annotations: %s" % repr(accs)
         for ref in record.dbxrefs :
             assert ref and ref == ref.strip(), \
@@ -341,6 +423,49 @@ for (t_format, t_alignment, t_filename, t_count) in test_files :
             #Expected to fail
             pass
 
+    # Check alphabets
+    for record in records :
+        base_alpha = Alphabet._get_base_alphabet(record.seq.alphabet)
+        assert isinstance(base_alpha, Alphabet.SingleLetterAlphabet)
+        if t_format in no_alpha_formats :
+            assert base_alpha == Alphabet.single_letter_alphabet # Too harsh?
+    if isinstance(base_alpha, Alphabet.ProteinAlphabet) :
+        good = protein_alphas
+        bad = dna_alphas + rna_alphas + nucleotide_alphas
+    elif isinstance(base_alpha, Alphabet.RNAAlphabet) :
+        good = nucleotide_alphas + rna_alphas
+        bad = protein_alphas + dna_alphas
+    elif isinstance(base_alpha, Alphabet.DNAAlphabet) :
+        good = nucleotide_alphas + dna_alphas
+        bad = protein_alphas + rna_alphas
+    elif isinstance(base_alpha, Alphabet.NucleotideAlphabet) :
+        good = nucleotide_alphas
+        bad = protein_alphas
+    else :
+        assert t_format in no_alpha_formats, "Got %s from %s file" \
+               % (repr(base_alpha), t_format)
+        good = protein_alphas + dna_alphas + rna_alphas + nucleotide_alphas
+        bad = []
+    for given_alpha in good :
+        #These should all work...
+        given_base = Alphabet._get_base_alphabet(given_alpha)
+        for record in SeqIO.parse(open(t_filename),t_format,given_alpha):
+            base_alpha = Alphabet._get_base_alphabet(record.seq.alphabet)
+            assert isinstance(base_alpha, given_base.__class__)
+            assert base_alpha == given_base
+        if t_count == 1 :
+            record = SeqIO.read(open(t_filename),t_format,given_alpha)
+            assert isinstance(base_alpha, given_base.__class__)
+            assert base_alpha == given_base
+    for given_alpha in bad :
+        #These should all fail...
+        try :
+            print SeqIO.parse(open(t_filename),t_format,given_alpha).next()
+            assert False, "Forcing wrong alphabet, %s, should fail (%s)" \
+                   % (repr(given_alpha), t_filename)
+        except ValueError :
+            pass
+    del good, bad, given_alpha, base_alpha
 
     if t_alignment :
         print "Testing reading %s format file %s as an alignment" \
@@ -375,7 +500,7 @@ print "(Note that some of these are expected to 'fail' and say why)"
 print
 for (records, descr) in test_records :
     print "Testing can write/read %s" % descr
-    for format in SeqIO._FormatToWriter.keys() :
+    for format in test_write_read_alignment_formats :
         print " Checking can write/read as '%s' format" % format
 
         #################
@@ -383,7 +508,8 @@ for (records, descr) in test_records :
         #################
         handle = StringIO()
         try :
-            SeqIO.write(records, handle, format)
+            c = SeqIO.write(records, handle, format)
+            assert c == len(records)
         except ValueError, e:
             #This is expected to happen on many of the examples.
             print " Failed: %s" % str(e)
@@ -406,7 +532,12 @@ for (records, descr) in test_records :
         #################
         assert len(new_records) == len(records)
         for record, new_record in zip(records, new_records) :
-            assert record.id == new_record.id
+            if format == "nexus" :
+                #The nexus parser will dis-ambiguate repeated record ids.
+                assert record.id == new_record.id or \
+                       new_record.id.startswith(record.id+".copy")
+            else :
+                assert record.id == new_record.id
             assert record.seq.tostring() == new_record.seq.tostring()
             #Using records_match(record, new_record) is too strict
 
